function showFlow(imgSeqFilename, indices, grid, threshold_c, symbol, imgFormat)
% Syntax: showFlow(imgSeqFilename, indices, grid, threshold_c, symbol, imgFormat)
%
% Description: Show flow on the image sequence.
%
% Author: Haiying Liu
%   Date: Oct 22, 2000
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

dbstop if error

msg = nargchk(2, 6, nargin);
if (~isempty(msg))
  error(strcat('ERROR:', msg));
end

clear msg;

if nargin < 3
  grid = 4;
end

if nargin < 4
  threshold_c = -1;
end

if nargin < 5
  symbol = '_';
end

if nargin < 6
  imgFormat = 'bmp';
end

%===========================================================================

% Define some macro
QUIVER_SCALE  = 1;
DO_SEPARATELY = 0;

% Directory for input/output data
IMAGE_DATA_DIR = '../../Data';
RESULT_DIR     = './Result';

dataPath       = [IMAGE_DATA_DIR, '/', imgSeqFilename, '/', imgSeqFilename];

flowFilename   = [imgSeqFilename, '_flow'];
flowPath       = [RESULT_DIR, '/', imgSeqFilename, '/', imgSeqFilename, ...
    '_', num2str(indices(1)), '_flow', '.mat'];
c_cPath        = [RESULT_DIR, '/', imgSeqFilename, '/', imgSeqFilename, ...
    '_', num2str(indices(1)), '_c_c', '.mat'];

% Load flow and confidence
flow = load(flowPath);
flow = flow.flow;

nRow = size(flow, 1);
nCol = size(flow, 2);

c_c  = load(c_cPath);
c_c  = c_c.c_c;

flowMask = c_c > threshold_c;
flow = flow .* cat(3, flowMask, flowMask);

% Show image.

% .. load image
imageName1 = [dataPath, symbol, num2str(indices(1)), '.', imgFormat];
img1       = double(imread(imageName1));

imageName2 = [dataPath, symbol, num2str(indices(2)), '.', imgFormat];
img2       = double(imread(imageName2));

rowShow = 1:grid:nRow;
colShow = 1:grid:nCol;
[meshX, meshY] = meshgrid(colShow, rowShow);
    
% .. image 1
figure;
show(wkeep(img1, [nRow, nCol]));

% .. image 1 with flow
figure;
show(wkeep(img1, [nRow, nCol]));
hold on;
quiver(meshX, meshY, ...
  flow(rowShow, colShow, 1), ...
  flow(rowShow, colShow, 2), QUIVER_SCALE);
axis equal;
axis tight;

name = [imgSeqFilename, '\_', num2str(indices(1)),]; 
title(name);

% .. image 2 with flow
figure;
show(wkeep(img2, [nRow, nCol]));
hold on;
quiver(meshX, meshY, ...
  flow(rowShow, colShow, 1), ...
  flow(rowShow, colShow, 2), QUIVER_SCALE);
axis equal;
axis tight;

name = [imgSeqFilename, '\_', num2str(indices(2)),]; 
title(name);

% .. flow without texture
figure;
quiver(meshX, meshY, ...
  flow(rowShow, colShow, 1), ...
  flow(rowShow, colShow, 2), QUIVER_SCALE);
axis equal;
axis ij;
axis tight;

% .. flow magnitude
figure
show(sqrt(flow(:,:,1) .* flow(:,:,1) + flow(:,:,2) .* flow(:,:,2)));
title('Flow magnitude');

% .. confidence
figure;
show(c_c);
title('Confidence');
