function colorRegionEvolution(allTraces, nbFrames, nbRows, nbCols)
%
% Usage: colorRegionEvolution(allTraces, nbFrames, nbRows, nbCols)
%
% Show evolution of regions as hierarchical mean shift analysis progresses
% by using artificial color, so that when two regions are merged, 
% the index of the merged parent region is the index of the largest child region

[nbPixels, nbTraces] = size(allTraces);

for i = 2:nbTraces 
% for i = 8:17 % start later
   i
   childTrace = allTraces(:,i-1);
   childWeights = findWeights(childTrace);
   parentTrace = allTraces(:,i);
   for j = 1:max(parentTrace);
		parentIndices = find(parentTrace == j); % scan each parent region
		childRegionPixels = childTrace(parentIndices);	
		bestIndex = findLongestRunIndex(childRegionPixels, childWeights);
		allTraces(parentIndices, i) = bestIndex; % replace parent index with bestIndex
   end; % for j
   labelBlock = reshape(allTraces(:,i), nbFrames, nbRows, nbCols);
   coloredPixBlock = colorEachLabel(labelBlock);
   figure, imagesc(squeeze(coloredPixBlock(6,:,:,:)));
   hold off;
end; % for i

% nbPicRows = nbTraces/4;
nbPicRows = 4;
figure, hold on;

count = 0;
for i = 2:17
   count = count + 1;
   labelBlock = reshape(allTraces(:,i), nbFrames, nbRows, nbCols);
   coloredPixBlock = colorEachLabel(labelBlock);
   subplot(nbPicRows, 4, count); 
   imagesc(squeeze(coloredPixBlock(6,:,:,:)));
   hold on;
end;
hold off;

function bestIndex = findLongestRunIndex(indexList, weights)
%
% Given a list of indices, find the indices that are repeated most often

	localWeights = weights(indexList);
	[maxWeight, maxLocaIndex] = max(localWeights);
	bestIndex = indexList(maxLocaIndex);
	




