function colorRegionEvolution(allTraces, nbFrames, nbRows, nbCols)
%
% Usage: colorRegionEvolution(allTraces, nbFrames, nbRows, nbCols)
%
% Show evolution of regions as hierarchical mean shift analysis progresses
% by using artificial color, so that when two regions are merged, 
% the index of the merged parent region is the index of the largest child region

[nbPixels, nbTraces] = size(allTraces);

for i = 2:nbTraces 
% for i = floor(nbTraces/2):nbTraces % start later
   childTrace = allTraces(:,i-1);
   childWeights = findWeights(childTrace);
   parentTrace = allTraces(:,i);
   for j = 1:max(parentTrace);
	parentIndices = find(parentTrace == j); % scan each parent region
	childRegionPixels = childTrace(parentIndices);
	childWeights = weights(childRegionPixels);
	maxWeight = max(childWeights);
	
	bestIndex = findLongestRunIndex(childRegionPixels, childWeights);
	allTraces(parentRegionPixels, i) = bestIndex; % replace parent index with bestIndex
   end; % for j
end; % for i

nbPicRows = nbTraces/4;
figure, hold on;

for i = 1:nbTraces 
   labelBlock = reshape(allTraces(:,i), nbFrames, nbRows, nbCols);
   coloredPixBlock = colorEachLabel(labelBlock);
   subplot(nbPicRows, 4, i); 
   imagesc(squeeze(colorPixBlock(6,:,:,:)));
   hold on;
end;

function bestIndex = findLongestRunIndex(indexList, weights)
%
% Given a list of indices, find the indices that are repeated most often

	localWeights = weights(indexList);
	[maxWeight, maxLocaIndex] = max(localWeights);
	bestIndex = indexList(maxLocalIndex);
	

function bestIndex = findLongestRunIndex0(indexList)
%
% Given a list of indices, find the indices that are repeated most often

	childRegionPixels = childTrace(parentIndices);
	childWeights = weights(childRegionPixels);
	maxWeight = max(childWeights);

	childIndices = sort(childRegionPixels);
	firstChildIndex = childIndices(1);
	runLength = 1; % length of sequence of identical indices
	bestIndex = firstChildIndex;
	maxRunLength = 1;
	for k = 2:length(childIndices)
	   nexChildIndex = childIndices(k);
	   if nextChildIndex == firstChildIndex
		runLength = runLength + 1;
	   else % we just came out of a run of identical indices
		if runLength > maxRunLength % check if finished run was better than previous
		   maxRunLength = runLength;
		   bestIndex = firstChildIndex;
		end;
		firstChildIndex = nextChildIndex;
		runLength = 1;
	   end; % else

		
		
coloredPixBlock = zeros(nbFrames, nbRows, nbCols, 3);

if 0
rgbLabels = label2rgb(labelBlock(:), palette); % use a 1D version of labelBlock
else
    labels = 1 + max(labelBlock(:)) - labelBlock(:);
rgbLabels = label2rgb(labels, palette); % use a 1D version of labelBlock
end;

for k = 1:3
  coloredPixBlock(:,:,:,k) =   reshape(rgbLabels(:,k), nbFrames, nbRows, nbCols);
end;

if 0
	for i = 1:nbFrames
        labels = squeeze(labelBlock(i,:,:));
        rgbLabels = label2rgb(labels, palette);
        coloredPixBlock(i,:,:,:) = rgbLabels;
	end;
end;

coloredPixBlock = uint8(coloredPixBlock);



