function [lineImageX, lineImageY, lineImageTop] = drawClusterLines3D(nbFrames, nbRows, nbCols, clusters, clusterSegments, clusterColors, showThickness)
%
% Usage: [lineImageX, lineImageY] = drawClusterLines3D(nbFrames, nbRows, nbCols, clusters, clusterSegments, clusterCOlors, showThickness)
%
% Draw projections of 3D lines on 2 images, x-t and y-t

[nbClusters, nbFeatures] = size(clusters);

xOffset  = ceil(sqrt(nbFrames^2 + nbCols^2)/2) + 1;
yOffset  = ceil(sqrt(nbFrames^2 + nbRows^2)/2) + 1;

lineImageX = zeros(nbFrames, nbCols, 3);
lineImageY = zeros(nbFrames, nbRows, 3);
lineImageTop = zeros(nbRows, nbCols, 3);

for i = 1:nbClusters
	segmentList = clusterSegments{i, 2};
	halfThickness = 0;
	if showThickness
	  halfThickness = round(sqrt(clusterSegments{i, 1}/pi)); % radius from area
	end; % if
	nbEndPts = size(segmentList,1);
	
%	projRadiantX = rotate90(clusters(i,1)) * pi/180;
%	projCosX = cos(projRadiantX);
%	projSinX = sin(projRadiantX);
	edgeRadiantX = clusters(i,1) * pi/180;
	edgeCosX = cos(edgeRadiantX);
	edgeSinX = sin(edgeRadiantX);
	lineX = clusters(i,3) - xOffset;
%	projRadiantY = rotate90(clusters(i,2)) * pi/180;
%	projCosY = cos(projRadiantY);
%	projSinY = sin(projRadiantY);
	edgeRadiantY = clusters(i,2) * pi/180;
	edgeCosY = cos(edgeRadiantY);
	edgeSinY = sin(edgeRadiantY);
	lineY = clusters(i,4) - yOffset;
	
	rgb = clusterColors(i,:);
	gotSegment = 0;
	endPtIndex = 1;

   for iFrame = 1:nbFrames
    if iFrame == segmentList(endPtIndex)
       gotSegment = ~gotSegment;
       newSegmentStarting = gotSegment; % set only at first point of segment
       endPtIndex = endPtIndex + 1;
	   if endPtIndex > nbEndPts;
	      break;
	   end;
    end; % if
    if gotSegment
%	lineRow = round(nbRows/2 + (lineY - projSinY * (iFrame - nbFrames/2))/projCosY);
%	lineCol = round(nbCols/2 + (lineX - projSinX * (iFrame - nbFrames/2))/projCosX);
	lineRow = round(nbRows/2 + (lineY + edgeCosY * (iFrame - nbFrames/2))/edgeSinY);
 	lineCol = round(nbCols/2 + (lineX + edgeCosX * (iFrame - nbFrames/2))/edgeSinX);
	
	if lineRow >0 & lineRow<nbRows+1 & lineCol >0 & lineCol<nbCols+1
		lineImageTop(lineRow, lineCol, :) = rgb;
	end; % if

        if ~newSegmentStarting % then prevLineCol is defined
            step = sign(lineRow-prevLineRow);
            for t = prevLineRow : step: lineRow % fill between the dots
             if t<nbRows+1 & t>0
                lineImageY(iFrame, t,:) = rgb;
              end; % if
	        end; % for t
            step = sign(lineCol-prevLineCol);
            for t = prevLineCol : step: lineCol % fill between the dots
             if t<nbCols+1 & t>0
                lineImageX(iFrame, t,:) = rgb;
              end; % if
	        end; % for t
        end;
		if showThickness
		    for t = lineRow-halfThickness : lineRow+halfThickness
	            if t<nbRows+1 & t>0
	                lineImageY(iFrame, t,:) = rgb;
	            end; % if
		    end; % for t
		    for t = lineCol-halfThickness : lineCol+halfThickness
	            if t<nbCols+1 & t>0
	                lineImageX(iFrame, t,:) = rgb;
	            end; % if
		    end; % for t
		end; % if showThickness
        prevLineRow = lineRow;
        prevLineCol = lineCol;
        newSegmentStarting = 0;
   	end; % if
   end; % for
end

lineImageX = uint8(lineImageX);
lineImageY = uint8(lineImageY);
lineImageTop = uint8(lineImageTop);
