function fusedTrace3D = fuseMaskedRegions(LImage, uImage, vImage, clusterTrace3D, vDefinedBlock, nbNeighbors)
%
% Assign pixels of ~vDefined the labels of neighbor pixels where optical flow v is defined
% where neighborhood is based on pixel color and position in each individual frame
%

[nbFrames, nbRows, nbCols] = size(clusterTrace3D);
fusedTrace3D = clusterTrace3D;
fusedTrace1 = clusterTrace3D;
fusedTrace2 = clusterTrace3D;

[frameCountBlock, rowCountBlock, colCountBlock] = makeCountBlocks(nbFrames, nbRows, nbCols);

L3D = reshape(LImage, nbFrames, nbRows, nbCols);
u3D = reshape(uImage, nbFrames, nbRows, nbCols);
v3D = reshape(vImage, nbFrames, nbRows, nbCols);

% importances = [1,1,0.01,0.01,0.01]; % change default later
importances = [1,1,1,1,1]; % change default later

for i = 1:nbFrames
	LIm = L3D(i,:,:); 
	uIm = u3D(i,:,:); 
	vIm = v3D(i,:,:);
	trace = squeeze(clusterTrace3D(i,:,:));
	xIm = colCountBlock(i,:,:);
	yIm = rowCountBlock(i,:,:);
	vDefined2D = vDefinedBlock(i,:,:);
	flowOK = vDefined2D(:);
	trace1 = trace(flowOK);
	features = [xIm(:), yIm(:), LIm(:), uIm(:), vIm(:)];
	[normalizedFeatures, ranges] = normalizeFeatures(features, importances);
	features1 = normalizedFeatures(flowOK,:);
	features2 = normalizedFeatures(~flowOK,:);


	treeData = nn_prepare(features1);
    
	[closestFeatures1, distances] = nn_search(features1, treeData, features2, nbNeighbors); % k-NN
    if nbNeighbors == 1 % NN
        labelsFlowNotOK = trace1(closestFeatures1);
    else
		nbPts = size(closestFeatures1, 1);
        labelsFlowNotOK = zeros(nbPts,1);
		for j = 1:nbPts
            localLabels = trace1(closestFeatures1(j,:));
            labelsFlowNotOK(j) = findBestNeighbor(localLabels, nbNeighbors);
		end % for j
	end % else
	fusedTrace1(i,:,:) = squeeze(trace);
	trace(~flowOK) = labelsFlowNotOK;
	trace2 = trace;
	trace2(flowOK) = 0;
	fusedTrace2(i,:,:) = trace2;
end

fusedTrace3D = fusedTrace1 + fusedTrace2;
slicePixelBlock(colorEachLabel3(fusedTrace1));
slicePixelBlock(colorEachLabel3(fusedTrace2));
slicePixelBlock(colorEachLabel3(fusedTrace3D));

