function smallRegionTrace = fuseSmallRegions(featureList, clusterCenters, ranges)

% Usage: smallRegionTrace = fuseSmallRegions(featureVects, clusterCenters)
%
% Label featureVects using close-by cluster centers
%
% normalize both clusterCenters and featureVects at same time using ranges
% nn_search fails on non-normalized data

lo = 1; hi = 2;
[nbClusters, nbFeatures] = size(clusterCenters);

featureVects = double(featureList);

for i = 1:nbFeatures
    compMin = ranges(i,lo);
    compMax = ranges(i,hi);
    delta = compMax - compMin;
    if delta ~= 0
		featureVects(:,i) = (featureVects(:,i) - compMin)/delta;
		clusterCenters(:,i) = (clusterCenters(:,i) - compMin)/delta;
    else
		disp('*** PROBLEM WITH DELTA ***');
		pause;
    end;
end; % for

treeData = nn_prepare(clusterCenters);

disp('Nearest neighbor search to fuse small region pixels to large cluster centers');
tic
[smallRegionTrace, distances] = nn_search(clusterCenters, treeData, featureVects, 1);
toc


