function smallRegionTrace = fuseSmallRegions2D(smallRegionFeatures, largeRegionFeatures, ranges)

% Usage: smallRegionTrace = fuseSmallRegions2D(featureList, clusterCenters)
%
% Label featureVects using close-by cluster centers
%
% normalize both clusterCenters and featureVects at same time using ranges
% nn_search fails on non-normalized data

lo = 1; hi = 2;
nbFeatures = size(smallRegionFeatures, 2);

% importances = [0.5,0.5,0.5, 1,1]; % relative importance of color and position
importances = [0.1,0.1,0.1, 1,1]; % relative importance of color and position
for i = 1:nbFeatures
    compMin = ranges(i,lo);
    compMax = ranges(i,hi);
    delta = compMax - compMin;
    if delta ~= 0
		smallRegionFeatures(:,i) = importances(i)*(smallRegionFeatures(:,i) - compMin)/delta;
		largeRegionFeatures(:,i) = importances(i)*(largeRegionFeatures(:,i) - compMin)/delta;
    else
		disp('*** PROBLEM WITH DELTA ***');
		pause;
    end;
end; % for

treeData = nn_prepare(largeRegionFeatures);

disp('Nearest neighbor search to fuse small region pixels to large regions');
tic
[smallRegionTrace, distances] = nn_search(largeRegionFeatures, treeData, smallRegionFeatures, 1);
toc


