function [fusedClusterTrace, bestClusterColors] = keepLargeRegions(featureList, clusterCenters, clusterTrace, clusterColors, ranges, minNbPix)
%
% Usage: [fusedClusterTrace, bestClusterColors] = keepLargeRegions(featureList, clusterCenters, clusterTrace, clusterColors, minNbPixels)
% Find clusters with more contributing pixels than minNbPixels
% and their corresponding colors
% then find all pixels that don't belong to the large clusters
% and assign them to the nearest neighbor clusters

bestIndexList = pickBestClusters2D(clusterCenters, clusterTrace, minNbPix);

bestClusterCenters = clusterCenters(bestIndexList,:);
bestClusterColors = clusterColors(bestIndexList,:);

clusterTraceBest = zeros(size(clusterCenters,1),1); % pixels pointing to small clusters will get zero trace
clusterTraceBest(bestIndexList) = [1:length(bestIndexList)];
bestClusterTrace = mergeTraces(clusterTrace, clusterTraceBest); % redirect pixels to new cluster centers
smallRegionFlags = (bestClusterTrace == 0);
[smallRegionFeatureList, smallRegionTrace] = keepNonZeroFeatures(featureList, smallRegionFlags); % reusing previous code, find pixels that don't belong to large clusters
% smallRegionLabels = fuseSmallRegions(smallRegionFeatureList, bestClusterCenters, ranges); % assign pixels to neighbor clusters
% smallRegionLabels = fuseSmallRegions(smallRegionFeatureList(:,4:5), bestClusterCenters(:,4:5), ranges(4:5,:)); % assign pixels to neighbor clusters using only proximity, not colors
smallRegionLabels = fuseSmallRegions(smallRegionFeatureList(:,1:3), bestClusterCenters(:,1:3), ranges(1:3,:)); % assign pixels to neighbor clusters using only proximity, not colors
smallRegionTrace = mergeTraces(smallRegionTrace, smallRegionLabels);

fusedClusterTrace = bestClusterTrace + smallRegionTrace; % when one labeling is zero the other is not and vice versa
