function [fusedClusterTrace, bestClusterColors] = keepLargeRegions2D(featureList, clusterCenters, clusterTrace, clusterColors, ranges, minNbPix)
%
% Usage: [fusedClusterTrace, bestClusterColors] = keepLargeRegions(featureList, clusterCenters, clusterTrace, clusterColors, ranges, minNbPixels)
% Find clusters with more contributing pixels than minNbPixels
% and their corresponding colors
% then find all pixels that don't belong to the large clusters
% and assign them to the nearest neighbor clusters

bestIndexList = pickBestClusters2D(clusterCenters, clusterTrace, minNbPix);

bestClusterColors = clusterColors(bestIndexList,:);

clusterTraceBest = zeros(size(clusterCenters,1),1); % pixels pointing to small clusters will get zero trace
clusterTraceBest(bestIndexList) = [1:length(bestIndexList)];
bestClusterTrace = mergeTraces(clusterTrace, clusterTraceBest); % redirect pixels to new cluster centers

smallRegionFlags = (bestClusterTrace == 0);
largeRegionFlags = ~smallRegionFlags;
largeRegionFeatureList = featureList;
largeRegionFeatureList(smallRegionFlags,:) = [];
largeRegionTrace = bestClusterTrace;
largeRegionTrace(smallRegionFlags) = [];
%  Trace to small region labels featureList in terms of pixels of small region
[smallRegionFeatureList, traceToSmallRegion] = keepNonZeroFeatures(featureList, smallRegionFlags); % reusing previous code, find pixels that don't belong to large clusters

smallRegionLabels = fuseSmallRegions2D(smallRegionFeatureList, largeRegionFeatureList, ranges); % assign pixels to largeRegion neighbor pixels
smallRegionLabels = mergeTraces(smallRegionLabels, largeRegionTrace); % from pointing to largeRegion pixels to pointing to clusters
% Note that the merging above is reversed compared to what we usually do
smallRegionTrace = mergeTraces(traceToSmallRegion, smallRegionLabels);

fusedClusterTrace = bestClusterTrace + smallRegionTrace; % when one labeling is zero, the other is not and vice versa
