function [largestClusters, largestClusterTrace] = keepLargestClusters(clusters, clusterTrace3D, minNbPix, doAddThickness)
%
% Usage: [largestClusters, largestClusterTrace] = keepLargestClusters(clusters, clusterTrace, minNbPix)
% 
% Keep only clusters whose numbers of pixels in occupied frames are more than minNbPix
% segments include pixel areas in their descriptions
%

	segments = findClusterSegments3D(clusters, clusterTrace3D); 
	
	bestIndexList = pickBestClusters3D(segments, minNbPix); % we need to sort regions by size
    
    nbLargeClusters = length(bestIndexList);
    largestSegments = segments(bestIndexList,:);
	
	% 		bestSegments = motionSegments(bestMotionIndexList,:);
	largestClusters = clusters(bestIndexList,:);
	
	clusterTraceBest = zeros(size(clusters,1),1); % pixels pointing to small clusters will get zero trace
	clusterTraceBest(bestIndexList) = [1:nbLargeClusters];
	largestClusterTrace = mergeTraces(clusterTrace3D(:), clusterTraceBest);

    if doAddThickness
        largestSegments = segments(bestIndexList,:);
        for i = 1:nbLargeClusters 
        	averageNbPixPerFrame = largestSegments{i,1}(1);
		    augmentedClusters(i,:) = [largestClusters(i,:), averageNbPixPerFrame];
        end
        largestClusters = augmentedClusters;
    end
