function featureVectors = makeFeatureVectors3D(clusterCenters, clusterSegments, projDistOffsetX, projDistOffsetY)

% Usage: featureVectors = makeFeatureVectors3D(clusterCenters, clusterSegments, projDistOffset)

% Add average nb of pixels on each row where original pixels point to the cluster as feature component
% so that features are: 
% 1. angleX of row slices of video stack from 1 to 180
% 2. angleY of column slices of video stack from 1 to 180
% 3. distanceX from center of slice to line of time structure (we remove the offset used to allow for sparse matrices)
% 4. distanceY from center of slice to line of time structure
% 5 & 6. R', G' so that objects with different illuminations are matched
% 7. Average number of pixels per occupied row

nbClusters = size(clusterCenters, 1);

for i = 1:nbClusters
	averageNbPixPerFrame = clusterSegments{i,1}(1);
	featureVectors(i,:) = [clusterCenters(i,:), averageNbPixPerFrame];
	fixedClusterCenters(i,1:2) = clusterCenters(i,1:2);
	featureVectors(i,3) = featureVectors(i,3) - projDistOffsetX; % can be positive or negative now
	featureVectors(i,4) = featureVectors(i,4) - projDistOffsetY; % can be positive or negative now
end
