function makeMPEGFromBMPFiles(path, prefix, firstFrame, lastFrame)
%
% Make MPEG movie from a sequence of bmp files called <prefix>00001.bmp, etc.
% The output is called <prefix>.mpg, in the same directory as ppm files

% 0. Generate ppm files from bmp files

fullInputName = [path, '/', prefix];
fullOutputName = [path, '/', prefix, '_'];

for i = firstFrame:lastFrame
	disp(i);
	inputImageName = [fullInputName, sprintf('%05d.bmp',i)];
	outputImageName = [fullOutputName, sprintf('%05d.ppm',i)];
	command = ['convert ', inputImageName, ' ', outputImageName];
	unix(command);
end	

outputFileName = [path, '/', prefix,'.mpg'];
% 1. Make template file for mpeg_encode

line{1} = 'PATTERN	 IBBPBB';
line{2} = ['OUTPUT	', outputFileName];
line{3} = 'BASE_FILE_FORMAT	 PPM';
line{4} = 'INPUT_CONVERT	 *';
line{5} = 'GOP_SIZE	 6';
line{6} = 'SLICES_PER_FRAME	 15';
line{7} = ' ';
line{7} = ['INPUT_DIR	 ', path];
line{8} = 'INPUT';
line{9} = [prefix,'_*.ppm    [',  sprintf('%05d',firstFrame),'-', sprintf('%05d',lastFrame),']'];
line{10} = 'END_INPUT';
line{11} = '    ';
line{12} = 'PIXEL	 HALF';
line{13} = 'RANGE	 10';
line{14} = 'PSEARCH_ALG	 LOGARITHMIC';
line{15} = 'BSEARCH_ALG	 CROSS2';
line{16} = 'IQSCALE	 4';
line{17} = 'PQSCALE	 8';
line{18} = 'BQSCALE	 15';
line{19} = 'REFERENCE_FRAME	 ORIGINAL';
line{20} = 'FORCE_ENCODE_LAST_FRAME';

templateName = [path, '/', prefix, 'Template.txt'];

% 2. Store template file:

fid = fopen(templateName,'w');
for k = 1:20
    fprintf(fid, '%s\n',char(line{k}))
end;

% 3. Make movie using mpeg_encode
command = ['mpeg_encode -realquiet ',templateName];
unix(command);
