function makeMPEGMovie(pixelBlock, fileName, scale)
%
% Usage: makeMPEGMovie(pixelBlock, fileName, scale)
%
% make an MPEG movie out of the frames in a pixel block and store it using given file name
%
% Input 1: pixelBlock, a 4D array. First dimension is number of frames. 
% Each frame is a 3D RGB image in uint8 format
% Input 2: Name chosen for output MPEG movie
% Input 3: multiplying scale for movie frames. 
% A scale of 4 produces movie frames 4 times the size of pixelBlock frames.
% Example of call: makeMPEGMovie(segmented3D, '/dfs/homes/daniel/MacTreeMeanShift3D_2/HandPaper/handLayer.mpg', 1);
%[nbFrames, nbRows, nbCols, nbColors] = size(pixelBlock);
[nbFrames, nbRows, nbCols] = size(pixelBlock);

prefix = strtok(fileName,'.');

% 1. Read each frame of pixel block and make a ppm file
for i= 1:nbFrames
    disp(i);
    rgbImage = imresize(squeeze(pixelBlock(i,:,:,:)), scale, 'bilinear');
    imageName = [prefix,'_', sprintf('%05d.ppm',i)];
    myPPMWrite(rgbImage, imageName);
end;

% 2. Make template file for mpeg_encode

line{1} = 'PATTERN	 IBBPBB';
line{2} = ['OUTPUT	', fileName];
line{3} = 'BASE_FILE_FORMAT	 PPM';
line{4} = 'INPUT_CONVERT	 *';
line{5} = 'GOP_SIZE	 6';
line{6} = 'SLICES_PER_FRAME	 15';
line{7} = ' ';
line{7} = 'INPUT_DIR	 ./';
line{8} = 'INPUT';
line{9} = [prefix,'_*.ppm','    [00001-', sprintf('%05d',nbFrames),']'];
line{10} = 'END_INPUT';
line{11} = '    ';
line{12} = 'PIXEL	 HALF';
line{13} = 'RANGE	 10';
line{14} = 'PSEARCH_ALG	 LOGARITHMIC';
line{15} = 'BSEARCH_ALG	 CROSS2';
line{16} = 'IQSCALE	 4';
line{17} = 'PQSCALE	 8';
line{18} = 'BQSCALE	 15';
line{19} = 'REFERENCE_FRAME	 ORIGINAL';
line{20} = 'FORCE_ENCODE_LAST_FRAME';

templateName = [prefix, 'Template.txt'];
fid = fopen(templateName,'w');
for k = 1:20
    fprintf(fid, '%s\n',char(line{k}))
end;

% 3. Make movie using mpeg_encode
command = ['mpeg_encode ',templateName];
unix(command);



