function x = showAllImages3D(rgb3D, allLinesImX, allLinesImY, ...
	  		bestLinesImX1, bestLinesImY1, bestLinesImX2, bestLinesImY2, bestLinesTop, ...
			angleMapX, angleMapY, projMapX, projMapY, segmented3D, angle3DX, angle3DY)


% Usage: x = showAllImages3D(rgb3D, allLinesImX, allLinesImY, ...
%	  		bestLinesImX1, bestLinesImY1, bestLinesImX2, bestLinesImY2, ...
%			angleMapX, angleMapY, projMapX, projMapY, segmented3D, angle3DX, angle3DY);

% Represent results on 2 figures

% 1. Initial analysis of inputs
% with angle map, and projection distance map

% 2. Final results, with initial rgb image,  
% color segmentation and angle segmentation
% line segments, best line segments.

[nbFrames, nbRows, nbCols, nbColors] = size(rgb3D);

halfNbFrames = round(nbFrames/2);
halfNbRows  = round(nbRows/2);
halfNbCols = round(nbCols/2);

    
% Original images
figure, subplot(2, 2, 1),
imshow(squeeze(rgb3D(1,:,:,:)));
hold on;
title('First image in stack');

subplot(2, 2, 2),
imshow(squeeze(rgb3D(nbFrames,:,:,:)));
hold on;
title('Last image in stack');

subplot(2, 2, 3),
imshow(squeeze(rgb3D(:,halfNbRows,:,:)));
hold on;
title('Space-time slice X');

subplot(2, 2, 4),
imshow(squeeze(rgb3D(:,:,halfNbCols,:)));
hold on;
title('Space-time slice Y');

hold off;

if 0
% Second image: angles and FON distances

	figure, subplot(2, 2, 1),
	imshow(squeeze(angleMapX(:,halfNbRows,:))),colorbar, colormap(hot);
	hold on;
	title('Edge angles X');
	
	subplot(2, 2, 2),
	imshow(squeeze(angleMapY(:,:,halfNbCols))),colorbar, colormap(hot);
	hold on;
	title('Edge angles Y');
	
	subplot(2, 2, 3),
	imshow(squeeze(projMapX(:,halfNbRows,:))),colorbar, colormap(hot);
	hold on;
	title('Foot-of-normal distances X');
	
	subplot(2, 2, 4),
	imshow(squeeze(projMapY(:,:,halfNbCols))),colorbar, colormap(hot);
	hold on;
	title('Foot-of-normal distances Y');
	
	hold off;
	
end; %if

% Third image: Results for segmentation

figure, subplot(2, 2, 1);
imshow(squeeze(segmented3D(:,halfNbRows,:,:))),
hold on;
title(' Color segmentation X');

subplot(2, 2, 2);
imshow(squeeze(segmented3D(:,:,halfNbCols,:))),
hold on;
title(' Color segmentation Y');

subplot(2, 2, 3);
imshow(squeeze(angle3DX(:,halfNbRows,:)),[]),colorbar;
colormap(hot);
hold on;
title('Angle segmentation X');

subplot(2, 2, 4);
imshow(squeeze(angle3DY(:,:,halfNbCols)),[]),colorbar;
colormap(hot);
hold on;
title('Angle segmentation Y');

hold off;

% Fourth image: Results for lines

figure, subplot(3, 2, 1);
imshow(squeeze(segmented3D(1,:,:,:))),
hold on;
title('Segmented first image');

subplot(3, 2, 2);
imshow(squeeze(segmented3D(nbFrames,:,:,:))),
hold on;
title('Segmented last image');

subplot(3, 2, 3);
imshow(bestLinesTop),
hold on;
title('Top view of line segments');

subplot(3, 2, 5);
imshow(bestLinesImX1),
hold on;
title('Best x-t line segments');

subplot(3, 2, 6);
imshow(bestLinesImY1),
hold on;
title('Best y-t line segments');


hold off;

