function clusterSegments = findClusterSegments3D(clusterCenters, clusterTrace3D)

% Usage: clusterSegments = findClusterSegments3D(clusterTrace3D, clusterCenters)

% Find the time intervals (rows of shotImage) 
% along which clusters are defined
% Use cluster trace to find only portions of cluster lines 
% for which there are contributing pixels in the same frame

[nbFrames, nbRows, nbCols] = size(clusterTrace3D);
nbClusters = size(clusterCenters,1);

% count nb of pixels per frame and per cluster
frameCountPerCluster = zeros(nbFrames, nbClusters); % 2D table

clusterSegments = cell(nbClusters,2); % count per frame and end point list

for i = 1:nbFrames
	for j = 1:nbRows
		for k = 1:nbCols
			clusterIndex = clusterTrace3D(i, j, k);
			if clusterIndex ~= 0
				frameCountPerCluster(i, clusterIndex) = frameCountPerCluster(i, clusterIndex) + 1;
			end; % if
		end;
	end;
end;

for iCluster = 1:nbClusters
	fullFrameCount = 0; % count non empty rows for each cluster
	pixelCount = 0;
	endPointList = [];
	gotSegment= 0;
	for iFrame = 1:nbFrames
		framePixelCount = frameCountPerCluster(iFrame, iCluster);
		pixelCount = pixelCount + framePixelCount;
		if framePixelCount > 0
			fullFrameCount = fullFrameCount + 1;
			if gotSegment==0
				endPointList = [endPointList; iFrame]; % segment started
				gotSegment = 1;
			end;
		else % framePixelCount == 0
			if gotSegment == 1
				endPointList = [endPointList; iFrame]; % segment ended in previous frame
				gotSegment = 0;
			end; % if
		end; % if
	end; % for
	
	if gotSegment == 1 % pixels down to last row
		endPointList = [endPointList; nbFrames+1]; % segment ended in previous row
	end; % if
    if fullFrameCount>0
	    pixelCountPerFrame = round(pixelCount/fullFrameCount); % average pixel count in occupied frames
    else
        pixelCountPerFrame = 0;
    end;
	clusterSegments{iCluster,1} = pixelCountPerFrame;
	clusterSegments{iCluster,2} = endPointList;
end; % for
			

