function [lineList, colorList, thicknessList] = listClusterLines3D(nbFrames, nbRows, nbCols, clusters, clusterSegments, clusterColors)
%
% Usage: [lineList, colorList] = listClusterLines3D(nbFrames, nbRows, nbCols, clusters, clusterSegments, clusterColors, showThickness)
%
% Make list of line segments and list of corresponding colors
% corresponding to each cluster center
% A line segment is given by x1 y1 z1 x2 y2 z2
% There can be several interrupted line segment for each cluster
% clusterSegments lists how many end points there are and at what frame they occur

[nbClusters, nbFeatures] = size(clusters);

xOffset  = ceil(sqrt(nbFrames^2 + nbCols^2)/2) + 1;
yOffset  = ceil(sqrt(nbFrames^2 + nbRows^2)/2) + 1;

lineList = [];
colorList = [];
thicknessList = [];
for i = 1:nbClusters
	segmentList = clusterSegments{i, 2};
	thickness = 2*round(sqrt(clusterSegments{i, 1}/pi)); % radius from area
    
    nbEndPts = size(segmentList,1); % make sure this is a column vector
	
	edgeRadiantX = clusters(i,1) * pi/180;
	edgeCosX = cos(edgeRadiantX);
	edgeSinX = sin(edgeRadiantX);
	lineX = clusters(i,3) - xOffset;
	edgeRadiantY = clusters(i,2) * pi/180;
	edgeCosY = cos(edgeRadiantY);
	edgeSinY = sin(edgeRadiantY);
	lineY = clusters(i,4) - yOffset;
	
	rgb = double(clusterColors(i,:))/255.0;
	
	nbSegs = nbEndPts/2; % nbEndPts should be even
	endPtIndex = 0;
	
	for iSeg = 1:nbSegs
		endPtIndex = endPtIndex + 1;
		iFrame1 = segmentList(endPtIndex); % starting pt of segment
		lineRow1 = round(nbRows/2 + (lineY + edgeCosY * (iFrame1 - nbFrames/2))/edgeSinY);
		lineCol1 = round(nbCols/2 + (lineX + edgeCosX * (iFrame1 - nbFrames/2))/edgeSinX);
		endPtIndex = endPtIndex + 1;
		iFrame2 = segmentList(endPtIndex);  % ending pt of segment
		lineRow2 = round(nbRows/2 + (lineY + edgeCosY * (iFrame2 - nbFrames/2))/edgeSinY);
		lineCol2 = round(nbCols/2 + (lineX + edgeCosX * (iFrame2 - nbFrames/2))/edgeSinX);
		lineSeg = [lineCol1, -lineRow1, -iFrame1, lineCol2, -lineRow2, -iFrame2+1]; % check if we can reverse z axis instead
		lineList = [lineList; lineSeg];
		colorList = [colorList; rgb];
        thicknessList = [thicknessList; thickness];
	end; % for iSeg
end; % for i cluster

