function [bestMovingClusters, bestMovingTrace, bestMovingIndices] = selectBestMovingClusters(clusters, trace, nbFrames, minNbPix, minAngle)

% Usage: [bestClusters, bestTrace] = selectBestMovingClusters(clusters, trace, nbFrames, minNbPix)
%
% keep only clusters with largest average nb of pix per row
% and discard clusters with motion angles less than min degrees away from 90 degrees in both X and Y directions

weights = findWeights(trace);
[sortedWeights, mappedIndices] = sort(-weights);
sortedWeights = - sortedWeights/nbFrames;
sortedClusters = clusters(mappedIndices,:); % mappedIndices are positions of unsorted clusters once they are sorted

bestWeights = sortedWeights(sortedWeights>=minNbPix);
nbBestClusters = length(bestWeights);
bestClusters = sortedClusters(1:nbBestClusters,:);
bestClusters = [bestClusters, bestWeights];
bestIndices = mappedIndices(1:nbBestClusters); % old labels for best clusters

clusterAnglesX = bestClusters(:,1);
clusterAnglesY = bestClusters(:,2);
lowBadAngle = 90 - minAngle; % bounds of angles to avoid
highBadAngle = 90 + minAngle;
maskForSmallAngles =    clusterAnglesX > lowBadAngle & ...
                        clusterAnglesX < highBadAngle & ...
                        clusterAnglesY > lowBadAngle & ...
                        clusterAnglesY < highBadAngle;
bestMovingClusters = bestClusters(~maskForSmallAngles,:);

bestMovingIndices = bestIndices(~maskForSmallAngles);

bestMovingTrace = zeros(size(trace));

nbBestMovingClusters = size(bestMovingClusters,1);


disp('Compute best moving trace');
if 1
    localTrace = zeros(size(clusters,1),1);
    localTrace(bestMovingIndices) = [1:nbBestMovingClusters];
    bestMovingTrace = mergeTraces(trace, localTrace);
else
	for i = 1:nbBestMovingClusters
        label = bestMovingIndices(i); % old labels of selected clusters
        goodIndices = find(trace == label); % pixels where those selected clusters are in trace
        bestMovingTrace(goodIndices) = i; % make these pixels point to new clusters in trace: WARNING: can't use RGB color with these labels
	end
end;

