function [angleMapX, angleMapY, vDefinedBlock] = makeAngleMaps2(grey3D)

% Usage: [angleMapX, angleMapY] = makeAngleMaps(grey3D, magnitudeThreshold)
% Compute optical flow using Lucas-Kanade algorithm
% Here we don't return optical flow itself. We commented out uBlock and vBlock

grey3D = double(grey3D);

% Apply spatio-temporal gaussian filter
if 0
gaussF = (exp(-[-5:5].^2/(2*1.5*1.5))/sqrt(2*pi))'; % gaussian with sigma 1.5

gaussF = gaussF/sum(gaussF); % normalizing

grey3D = filter(gaussF, 1, grey3D, [], 1);
grey3D(1:9,:,:) = [];% temporal dimension
grey3D = filter(gaussF, 1, grey3D, [], 2); 
grey3D(:,1:9,:) = [];% rows
grey3D = filter(gaussF, 1, grey3D, [], 3); 
grey3D(:,:,1:9) = [];% columns

else % smaller filter so there are some pixels left
gaussF = (exp(-[-2:2].^2/(2*1.0*1.0))/sqrt(2*pi))'; % gaussian with sigma 1

gaussF = gaussF/sum(gaussF); % normalizing

grey3D = filter(gaussF, 1, grey3D, [], 1); 
grey3D(1:4,:,:) = [];% temporal dimension
grey3D = filter(gaussF, 1, grey3D, [], 2); 
grey3D(:,1:4,:) = [];% rows
grey3D = filter(gaussF, 1, grey3D, [], 3); 
grey3D(:,:,1:4) = [];% columns

    
end
% Compute spatio-temporal derivatives
d = [-1, 8, 0, -8, 1]/12;
It = filter(d,1,grey3D,[],1); 
Iy = filter(d,1,grey3D,[],2);
Ix = filter(d,1,grey3D,[],3);

% Trim so that all blocks have same size
It(1:4,:,:) = []; Ix(1:4,:,:) = []; Iy(1:4,:,:) = [];
It(:,1:4,:) = []; Ix(:,1:4,:) = []; Iy(:,1:4,:) = [];
It(:,:,1:4) = []; Ix(:,:,1:4) = []; Iy(:,:,1:4) = []; 

[nbFrames, nbRows, nbCols] = size(It)
angleMapX = zeros(nbFrames, nbRows, nbCols); % motion fields
angleMapY = zeros(nbFrames, nbRows, nbCols); % if 0, no motion was found

if 0
% Compute weight table
w = [0.0625,0.25,0.375,0.25,0.0625];
W = w' * w;
W = W * W;
W = reshape(W,1,25);
W2 = diag(W);
for i = 1:nbFrames
	disp(i);
    for j = 3:nbRows-2
        for k = 3:nbCols-2
            if (norm([Ix(i,j,k);Iy(i,j,k)]) > 1) % only for large spatial derivatives
                Ax = reshape(Ix(i,j-2:j+2,k-2:k+2),1,25);
                Ay = reshape(Iy(i,j-2:j+2,k-2:k+2),1,25);
                A = [Ax; Ay]';
                b = - reshape(It(i,j-2:j+2,k-2:k+2),25,1);
                MF = pinv(A'*W2*A)*A'*W2*b;
                u = MF(1);
                v = MF(2);
                angleMapX(i,j,k) = acos(u/sqrt(1+u*u)) * 180/pi; % from 1 to 179    
                angleMapY(i,j,k) = acos(v/sqrt(1+v*v)) * 180/pi;   
           end
        end
    end
end;

else
IxIx=Ix.*Ix;
IyIy=Iy.*Iy;
IxIy=Ix.*Iy;
IxIt=Ix.*It;
IyIt=Iy.*It;

mask = [0.0625,0.25,0.375,0.25,0.0625];

% uBlock = zeros(nbFrames, nbRows, nbCols);
% vBlock = zeros(nbFrames, nbRows, nbCols);
vDefinedBlock = logical(zeros(nbFrames, nbRows, nbCols));

for i =1:nbFrames
    disp(i);
%    clear fIxIx fIyIy fIxIy fIxIt fIyIt;
    fIxIx(:,:) = IxIx(i,:,:); fIyIy(:,:) = IyIy(i,:,:); fIxIy(:,:) = IxIy(i,:,:);
    fIxIt(:,:) = IxIt(i,:,:); fIyIt(:,:) = IyIt(i,:,:);
    fIxIx=conv2(mask, mask, fIxIx, 'same');	
    fIyIy=conv2(mask, mask, fIyIy, 'same');	
    fIxIy=conv2(mask, mask, fIxIy, 'same');	
    fIxIt=conv2(mask, mask, fIxIt, 'same');	
    fIyIt=conv2(mask, mask, fIyIt, 'same');	

    det = fIxIx .* fIyIy - fIxIy .*fIxIy;
    rhsU = -fIxIt .* fIyIy + fIyIt .* fIxIy;
    rhsV = fIxIt .* fIxIy - fIyIt .* fIxIx;

    vDefined = logical(det ~= 0);
    u = zeros(nbRows, nbCols);
    v = zeros(nbRows, nbCols);
    u(vDefined) = rhsU(vDefined) ./ det(vDefined);
    v(vDefined) = rhsV(vDefined) ./ det(vDefined);
    angleMapX(i,:,:) = acos(u ./ sqrt(1 + u .* u)) * 180/pi; % from 1 to 179    
    angleMapY(i,:,:) = acos(v ./ sqrt(1 + v .* v)) * 180/pi;
%    uBlock(i,:,:) = u;
%    vBlock(i,:,:) = v;
    vDefinedBlock(i,:,:) = vDefined(:,:);
end
end % else if ~0

angleMapX = uint8(angleMapX(:,3:nbRows-2,3:nbCols-2));
angleMapY = uint8(angleMapY(:,3:nbRows-2,3:nbCols-2));

% uBlock = uBlock(:,3:nbRows-2,3:nbCols-2);
% vBlock = vBlock(:,3:nbRows-2,3:nbCols-2);
vDefinedBlock = vDefinedBlock(:,3:nbRows-2,3:nbCols-2);
