function filteredBlock = filter1D(pixBlock, filterVect, dim)
%
% filter pixBlock with filterVect along 1 dimension defined by dim
% using the filter function 
% For a filter vect of length 2 N + 1, this function shifts the filter result by N
% so we pad the images with N zeros, then filter, then crop the fronts by N 
% to get the same size as originals without pixel shifts
% Good registration of motion and color is essential for space-time segmentation
%

nbFilterElems = length(filterVect);
if mod(nbFilterElems-1, 2) ~= 0
	disp('*** length of filter must be odd ***');
	pause;
end;

shift = (nbFilterElems - 1)/2; % shift created by filter in filtering direction

[nbFrames, nbRows, nbCols] = size(pixBlock);

toggle = zeros(1, 3);
toggle(dim) = 1;

filteredBlock = zeros(nbFrames+toggle(1)*shift, nbRows+toggle(2)*shift, nbCols+toggle(3)*shift);
filteredBlock(1:nbFrames, 1:nbRows, 1:nbCols) = pixBlock;

filteredBlock = filter(filterVect, 1, filteredBlock, [], dim);

filteredBlock(1:toggle(1)*shift,:,:) = []; % only one of these crops the block
filteredBlock(:,1:toggle(2)*shift,:) = [];
filteredBlock(:,:,1:toggle(3)*shift) = [];
