function filteredBlock = filter3D(pixBlock, filterVect)
%
% filter pixBlock with filterVect in 3 dimensions
% using the filter function along each dimension in succession
% For a filter vect of length 2 N + 1, this function shifts the filter result by N
% so we pad the images with N zeros, then filter, then crop the fronts by N 
% to get the same size as originals without pixel shifts
% Good registration of motion and color is essential for space-time segmentation
%

nbFilterElems = length(filterVect);
if mod(nbFilterElems-1, 2) ~= 0
	disp('*** length of filter must be odd ***');
	pause;
end;

shift = (nbFilterElems - 1)/2; % shift created by filter in filtering direction

[nbFrames, nbRows, nbCols] = size(pixBlock);

filteredBlock = zeros(nbFrames+shift, nbRows+shift, nbCols+shift);
filteredBlock(1:nbFrames, 1:nbRows, 1:nbCols) = double(pixBlock);

filteredBlock = filter(filterVect, 1, filteredBlock, [], 1);
filteredBlock = filter(filterVect, 1, filteredBlock, [], 2);
filteredBlock = filter(filterVect, 1, filteredBlock, [], 3);

filteredBlock(1:shift,:,:) = [];
filteredBlock(:,1:shift,:) = [];
filteredBlock(:,:,1:shift) = [];
