function [angleMapX, angleMapY, vDefinedBlock] = makeAngleMaps(grey3D)

% Usage: [angleMapX, angleMapY, vDefinedBlock] = makeAngleMaps(grey3D)
% Compute optical flow using Lucas-Kanade algorithm
% Here we don't return optical flow itself, only space-time angles. 
% We commented out uBlock and vBlock

disp('Corrected alignments in derivative maps');
grey3D = double(grey3D);

% Apply spatio-temporal gaussian filter
if 0 % larger smoothing
gaussF = (exp(-[-5:5].^2/(2*1.5*1.5))/sqrt(2*pi))'; % gaussian with sigma 1.5

gaussF = gaussF/sum(gaussF); % normalizing

grey3D = filter(gaussF, 1, grey3D, [], 1);
grey3D(1:8,:,:) = [];% temporal dimension
grey3D = filter(gaussF, 1, grey3D, [], 2); 
grey3D(:,1:8,:) = [];% rows
grey3D = filter(gaussF, 1, grey3D, [], 3); 
grey3D(:,:,1:8) = [];% columns

else % smaller filter so there are some pixels left
gaussF = (exp(-[-2:2].^2/(2*1.0*1.0))/sqrt(2*pi))'; % gaussian with sigma 1

gaussF = gaussF/sum(gaussF); % normalizing

grey3D = filter(gaussF, 1, grey3D, [], 1); 
grey3D(1:4,:,:) = [];% temporal dimension
grey3D = filter(gaussF, 1, grey3D, [], 2); 
grey3D(:,1:4,:) = [];% rows
grey3D = filter(gaussF, 1, grey3D, [], 3); 
grey3D(:,:,1:4) = [];% columns

    
end
% Compute spatio-temporal derivatives

[nbFrames, nbRows, nbCols] = size(grey3D);
% Compute spatio-temporal derivatives
d = [-1, 8, 0, -8, 1]/12;

% Filter displaces pixel blocks by 2 pixels away from edge along fitering dimension
% Also we have to keep images aligned and of same size
It = filter(d,1,grey3D,[],1); 
It(1:4,:,:) = []; % 2 pixels off to realign image and 2 off to trim bad pixels
grey3D(nbFrames-1:nbFrames,:,:) = [];
grey3D(1:2,:,:) = []; % to realign

Iy = filter(d,1,grey3D,[],2);
Iy(:,1:4,:) = [];
It(:,nbRows-1:nbRows,:) = [];
It(:,1:2,:) = [];
grey3D(:,nbRows-1:nbRows,:) = [];
grey3D(:,1:2,:) = [];

Ix = filter(d,1,grey3D,[],3);
Ix(:,:,1:4) = [];
It(:,:,nbCols-1:nbCols) = [];
It(:,:,1:2) = [];
Iy(:,:,nbCols-1:nbCols) = [];
Iy(:,:,1:2) = [];

[nbFrames, nbRows, nbCols] = size(It);

angleMapX = zeros(nbFrames, nbRows, nbCols); % motion fields
angleMapY = zeros(nbFrames, nbRows, nbCols); % if 0, no motion was found

if 0
% Compute weight table
w = [0.0625,0.25,0.375,0.25,0.0625];
W = w' * w;
W = W * W;
W = reshape(W,1,25);
W2 = diag(W);
for i = 1:nbFrames
	disp(i);
    for j = 3:nbRows-2
        for k = 3:nbCols-2
            if (norm([Ix(i,j,k);Iy(i,j,k)]) > 1) % only for large spatial derivatives
                Ax = reshape(Ix(i,j-2:j+2,k-2:k+2),1,25);
                Ay = reshape(Iy(i,j-2:j+2,k-2:k+2),1,25);
                A = [Ax; Ay]';
                b = - reshape(It(i,j-2:j+2,k-2:k+2),25,1);
                MF = pinv(A'*W2*A)*A'*W2*b;
                u = MF(1);
                v = MF(2);
                angleMapX(i,j,k) = acos(u/sqrt(1+u*u)) * 180/pi; % from 1 to 179    
                angleMapY(i,j,k) = acos(v/sqrt(1+v*v)) * 180/pi;   
           end
        end
    end
end;

else
    disp('Fast method ');
IxIx=Ix.*Ix;
IyIy=Iy.*Iy;
IxIy=Ix.*Iy;
IxIt=Ix.*It;
IyIt=Iy.*It;

mask = [0.0625,0.25,0.375,0.25,0.0625]; % sums to 1

% uBlock = zeros(nbFrames, nbRows, nbCols);
% vBlock = zeros(nbFrames, nbRows, nbCols);
vDefinedBlock = logical(zeros(nbFrames, nbRows, nbCols));
lambda = zeros(2, nbRows, nbCols);

for i =1:nbFrames
    disp(i);
%    clear fIxIx fIyIy fIxIy fIxIt fIyIt;
    fIxIx(:,:) = IxIx(i,:,:); fIyIy(:,:) = IyIy(i,:,:); fIxIy(:,:) = IxIy(i,:,:);
    fIxIt(:,:) = IxIt(i,:,:); fIyIt(:,:) = IyIt(i,:,:);
    fIxIx=conv2(mask, mask, fIxIx, 'same');	
    fIyIy=conv2(mask, mask, fIyIy, 'same');	
    fIxIy=conv2(mask, mask, fIxIy, 'same');	
    fIxIt=conv2(mask, mask, fIxIt, 'same');	
    fIyIt=conv2(mask, mask, fIyIt, 'same');	

    det = fIxIx .* fIyIy - fIxIy .*fIxIy;
    rhsU = -fIxIt .* fIyIy + fIyIt .* fIxIy;
    rhsV = fIxIt .* fIxIy - fIyIt .* fIxIx;
    
	diff = fIxIx - fIyIy;
	tmp =(fIxIx + fIyIy)*0.5;
	disc = 0.5 * sqrt(diff.^2 + 4.*(fIxIy.^2));
	lambda(1,:,:) = tmp - disc;
    lambda(2,:,:) = tmp + disc;

    minLambda = squeeze(min(lambda,[], 1)); % can lambda be negative? check: no

    vDefined = logical(minLambda > 0.1); % threshold suggested by Barron in IJCV paper is 1
    
    u = zeros(nbRows, nbCols);
    v = zeros(nbRows, nbCols);
    u(vDefined) = rhsU(vDefined) ./ det(vDefined);
    v(vDefined) = rhsV(vDefined) ./ det(vDefined);
%   angleMapX(i,:,:) = acos(u ./ sqrt(1 + u .* u)) * 180/pi; % from 1 to 179    
%   angleMapY(i,:,:) = acos(v ./ sqrt(1 + v .* v)) * 180/pi;
    angleMapX(i,:,:) = 90 - atan(u ) * 180/pi; % from 1 to 179; faster than acos    
    angleMapY(i,:,:) = 90 - atan(v) * 180/pi;
% 	 uBlock(i,:,:) = u;
%    vBlock(i,:,:) = v;
    vDefinedBlock(i,:,:) = vDefined(:,:);
end
end % else if ~0

angleMapX = uint8(angleMapX(:,3:nbRows-2,3:nbCols-2));
angleMapY = uint8(angleMapY(:,3:nbRows-2,3:nbCols-2));

% uBlock = uBlock(:,3:nbRows-2,3:nbCols-2);
% vBlock = vBlock(:,3:nbRows-2,3:nbCols-2);
vDefinedBlock = vDefinedBlock(:,3:nbRows-2,3:nbCols-2);
