function [angleMapX, angleMapY, vDefined] = makeOpticalFlowAngleMaps(grey3D)

% Usage: [angleMapX, angleMapY, singVal] = makeOpticalFlowAngleMaps(grey3D)
% Compute optical flow using our edge method algorithm

grey3D = double(grey3D);

% Apply spatio-temporal gaussian filter
if 1 % larger smoothing
gaussF = (exp(-[-5:5].^2/(2*1.5*1.5))/sqrt(2*pi))'; % gaussian with sigma 1.5

gaussF = gaussF/sum(gaussF); % normalizing

grey3D = filter(gaussF, 1, grey3D, [], 1);
grey3D(1:10,:,:) = [];% temporal dimension
grey3D = filter(gaussF, 1, grey3D, [], 2); 
grey3D(:,1:10,:) = [];% rows
grey3D = filter(gaussF, 1, grey3D, [], 3); 
grey3D(:,:,1:10) = [];% columns

else % smaller filter so there are some pixels left
gaussF = (exp(-[-2:2].^2/(2*1.0*1.0))/sqrt(2*pi))'; % gaussian with sigma 1

gaussF = gaussF/sum(gaussF); % normalizing

grey3D = filter(gaussF, 1, grey3D, [], 1); 
grey3D(1:4,:,:) = [];% temporal dimension
grey3D = filter(gaussF, 1, grey3D, [], 2); 
grey3D(:,1:4,:) = [];% rows
grey3D = filter(gaussF, 1, grey3D, [], 3); 
grey3D(:,:,1:4) = [];% columns

    
end

[nbFrames, nbRows, nbCols] = size(grey3D);
frameMapDX = zeros(nbFrames, nbRows, nbCols); % gradient fields
frameMapDY = zeros(nbFrames, nbRows, nbCols); 
frameMapMag = zeros(nbFrames, nbRows, nbCols); % magnitude of gradient
rowMapDX = zeros(nbFrames, nbRows, nbCols); % 
rowMapDT = zeros(nbFrames, nbRows, nbCols); % 
rowMapMag = zeros(nbFrames, nbRows, nbCols); % magnitude of gradient
colMapDY = zeros(nbFrames, nbRows, nbCols); % 
colMapDT = zeros(nbFrames, nbRows, nbCols); % 
colMapMag = zeros(nbFrames, nbRows, nbCols); % magnitude of gradient

motionVect = zeros(nbFrames, nbRows, nbCols, 3);
rad45 = 45 * pi/180; cos45 = cos(rad45); sin45 = sin(rad45);
op_roberts = [1, 0; 0, -1] / sqrt(2);

for i = 1:nbFrames
   image = squeeze(grey3D(i,:,:));
   % Get gradient.
   bx = filter2(op_roberts, image, 'same');
   by = filter2(rot90(op_roberts), image, 'same');
   frameMapDX(i,:,:) = bx .* cos45 - by .* sin45;
   frameMapDY(i,:,:) = -(bx .* sin45 + by .* cos45);
end;
frameMapMag = sqrt(frameMapDX.*frameMapDX + frameMapDY.*frameMapDY);

for j = 1:nbRows
   image = squeeze(grey3D(:,j,:));
   % Get gradient.
   bx = filter2(op_roberts, image, 'same');
   by = filter2(rot90(op_roberts), image, 'same');
   rowMapDX(:,j,:) = bx .* cos45 - by .* sin45;
   rowMapDT(:,j,:) = -(bx .* sin45 + by .* cos45);
end;
rowMapMag = sqrt(rowMapDX.*rowMapDX + rowMapDT.*rowMapDT);

for k = 1:nbCols
   image = squeeze(grey3D(:,:,k));
   % Get gradient.
   bx = filter2(op_roberts, image, 'same');
   by = filter2(rot90(op_roberts), image, 'same');
   colMapDY(:,:,k) = bx .* cos45 - by .* sin45;
   colMapDT(:,:,k) = -(bx .* sin45 + by .* cos45);
end;
colMapMag = sqrt(colMapDY.*colMapDY + colMapDT.*colMapDT);

rowsWin = (rowMapMag > colMapMag);
colsWin = (rowMapMag <= colMapMag);

normalT1 = -frameMapDY .* rowMapDX; % cross-products to get normals to planes
normalX1 = frameMapDY.* rowMapDT;
normalY1 = frameMapDX .* rowMapDT;

normalT2 = frameMapDX .* colMapDY;
normalX2 = frameMapDY.* colMapDT;
normalY2 = -frameMapDX .* colMapDT;

normalT = rowsWin .* normalT1 + colsWin .* normalT2; % best combinations
normalX = rowsWin .* normalX1 + colsWin .* normalX2;
normalY = rowsWin .* normalY1 + colsWin .* normalY2;

delta = 2; % try 2

singularVal = zeros(nbFrames, nbRows, nbCols); % singular values

for i = 1:nbFrames
    disp(i);
    for j = 1:nbRows
        for k = 1:nbCols
		   rowRange =  max(j-delta,1):min(j+delta,nbRows);
		   colRange =  max(k-delta,1):min(k+delta,nbCols);
		   windowT = normalT(i, rowRange, colRange);
		   windowX = normalX(i, rowRange, colRange);
		   windowY = normalY(i, rowRange, colRange);
		   mat = [windowT(:), windowX(:), windowY(:)];
		   [U,S,V] = svd(mat);
           smallestSV = S(3,3);
		   motionVect(i,j,k,:) = V(:,3); % 3D vect of length 1
           singularVal(i,j,k) = smallestSV > 0.01;
	    end;
    end;
end;

vDefined = logical(singularVal > 0.1); % arbitrary 

for i = 1:nbFrames
	t = squeeze(motionVect(i,:,:,1));
	u = squeeze(motionVect(i,:,:,2));
	v = squeeze(motionVect(i,:,:,3));
    t(~vDefined(i,:,:)) = 1; % to prevent divisions by zero
    u(~vDefined(i,:,:)) = 0;
    v(~vDefined(i,:,:)) = 0;
    
 % 	angleMapX(i,:,:) = acos(u ./ sqrt(t .* t + u .* u)) * 180/pi; % from 1 to 179    
%	angleMapY(i,:,:) = acos(v ./ sqrt(t .* t + v .* v)) * 180/pi;
	angleMapX(i,:,:) = 90 - atan(u ./ t) * 180/pi; % from 1 to 179    
	angleMapY(i,:,:) = 90 - atan(v ./ t) * 180/pi;
end;

angleMapX(nbFrames-delta:nbFrames, :,:) = [];
angleMapX(:,nbRows-delta:nbRows,:) = [];
angleMapX(:,:,nbCols-delta:nbCols) = [];
angleMapY(nbFrames-delta:nbFrames, :,:) = [];
angleMapY(:,nbRows-delta:nbRows,:) = [];
angleMapY(:,:,nbCols-delta:nbCols) = [];
singularVal(nbFrames-delta:nbFrames, :,:) = [];
singularVal(:,nbRows-delta:nbRows,:) = [];
singularVal(:,:,nbCols-delta:nbCols) = [];

angleMapX = uint8(angleMapX);
angleMapY = uint8(angleMapY);

vDefined = singularVal > 1; % arbitrary 


	


	   
   


