function [angleMapX, angleMapY, vDefinedBlock] = makeOpticalFlowAngleMaps2(grey3D)

% Usage: [angleMapX, angleMapY, singVal] = makeOpticalFlowAngleMaps(grey3D)
% Compute optical flow using our edge method algorithm

grey3D = double(grey3D);

% Apply spatio-temporal gaussian filter
if 1 % larger smoothing
gaussF = (exp(-[-5:5].^2/(2*1.5*1.5))/sqrt(2*pi))'; % gaussian with sigma 1.5

gaussF = gaussF/sum(gaussF); % normalizing

grey3D = filter(gaussF, 1, grey3D, [], 1);
grey3D(1:8,:,:) = [];% temporal dimension
grey3D = filter(gaussF, 1, grey3D, [], 2); 
grey3D(:,1:8,:) = [];% rows
grey3D = filter(gaussF, 1, grey3D, [], 3); 
grey3D(:,:,1:8) = [];% columns

else % smaller filter so there are some pixels left
gaussF = (exp(-[-2:2].^2/(2*1.0*1.0))/sqrt(2*pi))'; % gaussian with sigma 1

gaussF = gaussF/sum(gaussF); % normalizing

grey3D = filter(gaussF, 1, grey3D, [], 1); 
grey3D(1:4,:,:) = [];% temporal dimension
grey3D = filter(gaussF, 1, grey3D, [], 2); 
grey3D(:,1:4,:) = [];% rows
grey3D = filter(gaussF, 1, grey3D, [], 3); 
grey3D(:,:,1:4) = [];% columns

    
end

[nbFrames, nbRows, nbCols] = size(grey3D);
frameMapDX = zeros(nbFrames, nbRows, nbCols); % gradient fields
frameMapDY = zeros(nbFrames, nbRows, nbCols); 
frameMapMag = zeros(nbFrames, nbRows, nbCols); % magnitude of gradient
rowMapDX = zeros(nbFrames, nbRows, nbCols); % 
rowMapDT = zeros(nbFrames, nbRows, nbCols); % 
rowMapMag = zeros(nbFrames, nbRows, nbCols); % magnitude of gradient
colMapDY = zeros(nbFrames, nbRows, nbCols); % 
colMapDT = zeros(nbFrames, nbRows, nbCols); % 
colMapMag = zeros(nbFrames, nbRows, nbCols); % magnitude of gradient

rad135 = 135 * pi/180; cos135 = cos(rad135); sin135 = sin(rad135);
op_roberts = [1, 0; 0, -1] / sqrt(2);

for i = 1:nbFrames
   image = squeeze(grey3D(i,:,:));
   % Get gradient.
   bx = filter2(op_roberts, image, 'same');
   by = filter2(rot90(op_roberts), image, 'same');
   frameMapDX(i,:,:) = bx .* cos135 - by .* sin135;
   frameMapDY(i,:,:) = -(bx .* sin135 + by .* cos135);
end;

for j = 1:nbRows
   image = squeeze(grey3D(:,j,:));
   % Get gradient.
   bx = filter2(op_roberts, image, 'same');
   by = filter2(rot90(op_roberts), image, 'same');
   rowMapDX(:,j,:) = bx .* cos135 - by .* sin135;
   rowMapDT(:,j,:) = -(bx .* sin135 + by .* cos135);
end;

for k = 1:nbCols
   image = squeeze(grey3D(:,:,k));
   % Get gradient.
   bx = filter2(op_roberts, image, 'same');
   by = filter2(rot90(op_roberts), image, 'same');
   colMapDY(:,:,k) = bx .* cos135 - by .* sin135;
   colMapDT(:,:,k) = -(bx .* sin135 + by .* cos135);
end;

rowsWinIt = abs(rowMapDT)>abs(colMapDT);
It = rowsWinIt .* rowMapDT + (~rowsWinIt) .* colMapDT;

rowsWinIx = abs(rowMapDX)>abs(frameMapDX);
Ix = rowsWinIx .* rowMapDX + (~rowsWinIx) .* frameMapDX;

colsWinIy = abs(colMapDY)>abs(frameMapDY);
Iy = colsWinIy .* colMapDY + (~colsWinIy) .* frameMapDY;

IxIx=Ix.*Ix;
IyIy=Iy.*Iy;
IxIy=Ix.*Iy;
IxIt=Ix.*It;
IyIt=Iy.*It;

mask = [0.0625,0.25,0.375,0.25,0.0625];

% uBlock = zeros(nbFrames, nbRows, nbCols);
% vBlock = zeros(nbFrames, nbRows, nbCols);
vDefinedBlock = logical(zeros(nbFrames, nbRows, nbCols));
lambda = zeros(2, nbRows, nbCols);

for i =1:nbFrames
    disp(i);
%    clear fIxIx fIyIy fIxIy fIxIt fIyIt;
    fIxIx(:,:) = IxIx(i,:,:); fIyIy(:,:) = IyIy(i,:,:); fIxIy(:,:) = IxIy(i,:,:);
    fIxIt(:,:) = IxIt(i,:,:); fIyIt(:,:) = IyIt(i,:,:);
    fIxIx=conv2(mask, mask, fIxIx, 'same');	
    fIyIy=conv2(mask, mask, fIyIy, 'same');	
    fIxIy=conv2(mask, mask, fIxIy, 'same');	
    fIxIt=conv2(mask, mask, fIxIt, 'same');	
    fIyIt=conv2(mask, mask, fIyIt, 'same');	

    det = fIxIx .* fIyIy - fIxIy .*fIxIy;
    rhsU = -fIxIt .* fIyIy + fIyIt .* fIxIy;
    rhsV = fIxIt .* fIxIy - fIyIt .* fIxIx;
    
	diff = fIxIx - fIyIy;
	tmp =(fIxIx + fIyIy)*0.5;
	disc = 0.5 * sqrt(diff.^2 + 4.*(fIxIy.^2));
	lambda(1,:,:) = tmp - disc;
    lambda(2,:,:) = tmp + disc;

    minLambda = squeeze(min(lambda,[], 1)); % can lambda be negative? check

    vDefined = logical(minLambda > 0.1); % threshold suggested by Barron in IJCV paper is 1
    
    u = zeros(nbRows, nbCols);
    v = zeros(nbRows, nbCols);
    u(vDefined) = rhsU(vDefined) ./ det(vDefined);
    v(vDefined) = rhsV(vDefined) ./ det(vDefined);
 %   angleMapX(i,:,:) = acos(u ./ sqrt(1 + u .* u)) * 180/pi; % from 1 to 179    
 %   angleMapY(i,:,:) = acos(v ./ sqrt(1 + v .* v)) * 180/pi;
    angleMapX(i,:,:) = 90 - atan(u ) * 180/pi; % from 1 to 179; faster than acos    
	angleMapY(i,:,:) = 90 - atan(v) * 180/pi;
% 	 uBlock(i,:,:) = u;
%    vBlock(i,:,:) = v;
    vDefinedBlock(i,:,:) = vDefined(:,:);
end

angleMapX = uint8(angleMapX(:,3:nbRows-2,3:nbCols-2));
angleMapY = uint8(angleMapY(:,3:nbRows-2,3:nbCols-2));

% uBlock = uBlock(:,3:nbRows-2,3:nbCols-2);
% vBlock = vBlock(:,3:nbRows-2,3:nbCols-2);
vDefinedBlock = vDefinedBlock(:,3:nbRows-2,3:nbCols-2);

	   
   


