function [projMapX, projMapY] = makeProjMaps(angleMapX, angleMapY, projDistOffsetX, projDistOffsetY, vDefined)

% Usage: [projMapX, projMapY] = makeProjMaps(angleX, angleY, projDistOffsetX, projDistOffsetY, vDefined)
% Find images of projection distances

[nbFrames, nbRows, nbCols] = size(vDefined);
projMapX = zeros(nbFrames, nbRows, nbCols);
projMapY = zeros(nbFrames, nbRows, nbCols);

for iFrame = 1:nbFrames
   for iRow = 1:nbRows
	for iCol = 1:nbCols
	    if vDefined(iFrame,iRow,iCol)

		   edgeAngleX = angleMapX(iFrame, iRow, iCol);

		   radiantX = double(edgeAngleX) * pi/180;
		   edgeCosX = cos(radiantX);
		   edgeSinX = sin(radiantX); % we consider the vector perpendicular to (cos, sin)
		   projDistX = projDistOffsetX + edgeSinX*(iCol-nbCols/2) - edgeCosX*(iFrame-nbFrames/2);
		   
		   projMapX(iFrame, iRow, iCol) = projDistX;

		   edgeAngleY = angleMapY(iFrame, iRow, iCol);

		   radiantY = double(edgeAngleY) * pi/180;
		   edgeCosY = cos(radiantY);
		   edgeSinY = sin(radiantY); % we consider the vector perpendicular to (cos, sin)
		   projDistY = projDistOffsetY + edgeSinY*(iRow-nbRows/2) - edgeCosY*(iFrame-nbFrames/2);

		   projMapY(iFrame, iRow, iCol) = projDistY;

	    end; % if
	end; % for
   end; % for
end; % for

projMapX = uint8(round(projMapX));
projMapY = uint8(round(projMapY));
