function [angleMapX, angleMapY, vDefined] = makeSVDAngleMaps(grey3D)

% Usage: [angleMapX, angleMapY, singVal] = makeOpticalFlowAngleMaps(grey3D)
% Compute optical flow using our edge method algorithm

grey3D = double(grey3D);

% Apply spatio-temporal gaussian filter
if 1 % larger smoothing
gaussF = (exp(-[-5:5].^2/(2*1.5*1.5))/sqrt(2*pi))'; % gaussian with sigma 1.5

gaussF = gaussF/sum(gaussF); % normalizing

grey3D = filter(gaussF, 1, grey3D, [], 1);
grey3D(1:10,:,:) = [];% temporal dimension
grey3D = filter(gaussF, 1, grey3D, [], 2); 
grey3D(:,1:10,:) = [];% rows
grey3D = filter(gaussF, 1, grey3D, [], 3); 
grey3D(:,:,1:10) = [];% columns

else % smaller filter so there are some pixels left
gaussF = (exp(-[-2:2].^2/(2*1.0*1.0))/sqrt(2*pi))'; % gaussian with sigma 1

gaussF = gaussF/sum(gaussF); % normalizing

grey3D = filter(gaussF, 1, grey3D, [], 1); 
grey3D(1:4,:,:) = [];% temporal dimension
grey3D = filter(gaussF, 1, grey3D, [], 2); 
grey3D(:,1:4,:) = [];% rows
grey3D = filter(gaussF, 1, grey3D, [], 3); 
grey3D(:,:,1:4) = [];% columns

    
end

% Compute spatio-temporal derivatives
[nbFrames, nbRows, nbCols] = size(grey3D);
% Compute spatio-temporal derivatives
d = [-1, 8, 0, -8, 1]/12;

% Filter displaces pixel blocks by 2 pixels away from edge along fitering dimension
% Also we have to keep images aligned and of same size
It = filter(d,1,grey3D,[],1); 
It(1:4,:,:) = []; % 2 pixels off to realign image and 2 off to trim bad pixels
grey3D(nbFrames-1:nbFrames,:,:) = [];
grey3D(1:2,:,:) = []; % to realign

Iy = filter(d,1,grey3D,[],2);
Iy(:,1:4,:) = [];
It(:,nbRows-1:nbRows,:) = [];
It(:,1:2,:) = [];
grey3D(:,nbRows-1:nbRows,:) = [];
grey3D(:,1:2,:) = [];

Ix = filter(d,1,grey3D,[],3);
Ix(:,:,1:4) = [];
It(:,:,nbCols-1:nbCols) = [];
It(:,:,1:2) = [];
Iy(:,:,nbCols-1:nbCols) = [];
Iy(:,:,1:2) = [];

[nbFrames, nbRows, nbCols] = size(It);

angleMapX = zeros(nbFrames, nbRows, nbCols); % motion fields
angleMapY = zeros(nbFrames, nbRows, nbCols); % if 0, no motion was found
singVal3 = zeros(nbFrames, nbRows, nbCols); % if 0, no motion was found
singVal2 = zeros(nbFrames, nbRows, nbCols); % if 0, no motion was found
singVal1 = zeros(nbFrames, nbRows, nbCols); % if 0, no motion was found

mask = [0.0625,0.25,0.375,0.25,0.0625];

% uBlock = zeros(nbFrames, nbRows, nbCols);
% vBlock = zeros(nbFrames, nbRows, nbCols);
vDefined = logical(zeros(nbFrames, nbRows, nbCols));
delta = 2; % try 2
gaussF = (exp(-[-2:2].^2/(2*1.0*1.0))/sqrt(2*pi))'; % gaussian with sigma 1
gaussF = gaussF/sum(gaussF); % normalizing
gauss2 = gaussF * gaussF';

for i =1:nbFrames
    disp(i);
    fIx(:,:) = Ix(i,:,:); fIy(:,:) = Iy(i,:,:); fIt(:,:) = It(i,:,:);
%     fIx=conv2(mask, mask, fIx, 'same');	
%     fIy=conv2(mask, mask, fIy, 'same');	
%     fIt=conv2(mask, mask, fIt, 'same');	

    for j = 1:nbRows
        for k = 1:nbCols
		   rowRange =  max(j-delta,1):min(j+delta,nbRows);
		   colRange =  max(k-delta,1):min(k+delta,nbCols);
           if 0
			   windowT = fIt(rowRange, colRange);
			   windowX = fIx(rowRange, colRange);
			   windowY = fIy(rowRange, colRange);
           else
               weights = gauss2(rowRange-j+3, colRange-k+3);
			   windowT = weights .* fIt(rowRange, colRange); % weight centered vectors more
			   windowX = weights .* fIx(rowRange, colRange);
			   windowY = weights .* fIy(rowRange, colRange);
           end
		   mat = [windowT(:), windowX(:), windowY(:)];
		   [UU,SS,VV] = svd(mat,0); % find best normal to set of gradient vectors
%            singVal3(i,j,k) = SS(3,3)/SS(1,1);
%            singVal3(i,j,k) = SS(3,3);
%            singVal2(i,j,k) = SS(2,2)/SS(1,1);
%            singVal2(i,j,k) = SS(2,2);
%            singVal1(i,j,k) = SS(1,1);
%            SS(3,3)
  		     normal = VV(:,3); % 3D vect of length 1
             vDefined(i,j,k) = 1;
%              maxSingVal = 0.1;
             maxSingVal = 1;
            if 1
                vDefined(i,j,k) = SS(3,3) < maxSingVal; % larger is easier
            else
                if SS(3,3) > maxSingVal
                    initNormal = normal;
                    [normal, isConverged] = reduceSingVal(mat, initNormal, maxSingVal);
                    vDefined(i,j,k) = isConverged;
                    t = normal(1); u = normal(2); v = normal(3);
                end
            end
           t = normal(1); u = normal(2); v = normal(3);
           if vDefined(i,j,k)
     		   angleMapX(i,j,k) = 90 - atan(u ./ t) * 180/pi; % from 1 to 179    
			   angleMapY(i,j,k) = 90 - atan(v ./ t) * 180/pi;
           else
               angleMapX(i,j,k) = 0;
               angleMapY(i,j,k) = 0;
           end; % if
	    end;
    end;
end;

if 0
	confidence = singVal2 ./(singVal3+0.001);
	max0 = max(confidence(:)); min0 = min(confidence(:));
	confVal = uint8(255 * (confidence - min0)/(max0-min0));
	slicePixelBlock(confVal);
	
	max3 = max(singVal3(:)); min3 = min(singVal3(:));
	singVal3 = uint8(255 * (singVal3 - min3)/(max3-min3));
	slicePixelBlock(singVal3);
	
	max2 = max(singVal2(:)); min2 = min(singVal2(:));
	singVal2 = uint8(255 * (singVal2 - min2)/(max2-min2));
	slicePixelBlock(singVal2);
	
	max1 = max(singVal1(:)); min1 = min(singVal1(:));
	singVal1 = uint8(255 * (singVal1 - min1)/(max1-min1));
	slicePixelBlock(singVal1);
end
% slicePixelBlock(grey3D);

if 0 % check if necessary
	angleMapX(nbFrames-delta:nbFrames, :,:) = [];
	angleMapX(:,nbRows-delta:nbRows,:) = [];
	angleMapX(:,:,nbCols-delta:nbCols) = [];
	angleMapY(nbFrames-delta:nbFrames, :,:) = [];
	angleMapY(:,nbRows-delta:nbRows,:) = [];
	angleMapY(:,:,nbCols-delta:nbCols) = [];
	singularVal(nbFrames-delta:nbFrames, :,:) = [];
	singularVal(:,nbRows-delta:nbRows,:) = [];
	singularVal(:,:,nbCols-delta:nbCols) = [];
end % if 0

angleMapX = uint8(angleMapX);
slicePixelBlock(angleMapX);

angleMapY = uint8(angleMapY);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	   
   


