function [angleMapX, vDefined] = makeSVDAngleMaps2D(grey3D)

% Compute optical flow using our edge method algorithm

grey3D = double(grey3D);

% Apply spatio-temporal gaussian filter
if 1 % larger smoothing
gaussF = (exp(-[-5:5].^2/(2*1.5*1.5))/sqrt(2*pi))'; % gaussian with sigma 1.5

gaussF = gaussF/sum(gaussF); % normalizing

grey3D = filter(gaussF, 1, grey3D, [], 1);
grey3D(1:10,:,:) = [];% temporal dimension
grey3D = filter(gaussF, 1, grey3D, [], 2); 
grey3D(:,1:10,:) = [];% rows
grey3D = filter(gaussF, 1, grey3D, [], 3); 
grey3D(:,:,1:10) = [];% columns

else % smaller filter so there are some pixels left
gaussF = (exp(-[-2:2].^2/(2*1.0*1.0))/sqrt(2*pi))'; % gaussian with sigma 1

gaussF = gaussF/sum(gaussF); % normalizing

grey3D = filter(gaussF, 1, grey3D, [], 1); 
grey3D(1:4,:,:) = [];% temporal dimension
grey3D = filter(gaussF, 1, grey3D, [], 2); 
grey3D(:,1:4,:) = [];% rows
grey3D = filter(gaussF, 1, grey3D, [], 3); 
grey3D(:,:,1:4) = [];% columns

    
end

[nbFrames, nbRows, nbCols] = size(grey3D);
% Compute spatio-temporal derivatives
d = [-1, 8, 0, -8, 1]/12;

% Filter displaces pixel blocks by 2 pixels away from edge along fitering dimension
% Also we have to keep images aligned and of same size
It = filter(d,1,grey3D,[],1); 
It(1:4,:,:) = []; % 2 pixels off to realign image and 2 off to trim bad pixels
grey3D(nbFrames-1:nbFrames,:,:) = [];
grey3D(1:2,:,:) = []; % to realign

Iy = filter(d,1,grey3D,[],2);
Iy(:,1:4,:) = [];
It(:,nbRows-1:nbRows,:) = [];
It(:,1:2,:) = [];
grey3D(:,nbRows-1:nbRows,:) = [];
grey3D(:,1:2,:) = [];

Ix = filter(d,1,grey3D,[],3);
Ix(:,:,1:4) = [];
It(:,:,nbCols-1:nbCols) = [];
It(:,:,1:2) = [];
Iy(:,:,nbCols-1:nbCols) = [];
Iy(:,:,1:2) = [];

[nbFrames, nbRows, nbCols] = size(It);

angleMapX = zeros(nbFrames, nbRows, nbCols); % motion fields
angleMapY = zeros(nbFrames, nbRows, nbCols); % if 0, no motion was found

mask = [0.0625,0.25,0.375,0.25,0.0625];

% uBlock = zeros(nbFrames, nbRows, nbCols);
% vBlock = zeros(nbFrames, nbRows, nbCols);
vDefined = logical(zeros(nbFrames, nbRows, nbCols));
delta = 2; % try 2

for i =1:nbFrames
    disp(i);
    fIx(:,:) = Ix(i,:,:); fIy(:,:) = Iy(i,:,:); fIt(:,:) = It(i,:,:);
%   fIx=conv2(mask, mask, fIx, 'same');	
%   fIy=conv2(mask, mask, fIy, 'same');	
%   fIt=conv2(mask, mask, fIt, 'same');	

    for j = 1:nbRows
        for k = 1:nbCols
           if 1
               
			   rowRange =  max(j-delta,1):min(j+delta,nbRows);
			   colRange =  max(k-delta,1):min(k+delta,nbCols);
			   windowT = fIt(rowRange, colRange);
			   windowX = fIx(rowRange, colRange);
			   mat = [windowT(:), windowX(:)];
			   [U,S,V] = svd(mat); % find best normal to set of gradient vectors
			   t= V(1,2); u = V(2,2); % 2D vect of length 1
               vDefined(i,j,k) = abs(t) > 0;
               if vDefined(i,j,k)
         		   angleMapX(i,j,k) = 90 - atan(u ./ t) * 180/pi; % from 1 to 179    
               else
                   angleMapX(i,j,k) = 90;
               end; % if
           else
                t = fIt(j, k);
                u = fIx(j, k);
                vDefined(i,j,k) = abs(t)>0;
                if vDefined(i,j,k)
         		   angleMapX(i,j,k) = 90 - atan(u ./ t) * 180/pi; % from 1 to 179    
               else
                   angleMapX(i,j,k) = 90;
               end; % if
           end;
	    end;
    end;
end;


angleMapX = uint8(angleMapX);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	   
   


