function [angleMapX, angleMapY, vDefined] = makeSVDAngleMapsFromColors(aBlock, bBlock, cBlock)

% Usage: [angleMapX, angleMapY, singVal] =  makeSVDAngleMapsFromColors(aBlock, bBlock, cBlock)
% We use L*u*v color space
% We replace L,U,V bynotation a, b, c to avoid confusion with optical flow components u & v
% Compute optical flow from color video stack using our SVD algorithm

disp(' SVD method for computing optical flow ');

[nbFrames, nbRows, nbCols] = size(aBlock);
uBlock = zeros(nbFrames, nbRows, nbCols); % motion fields
vBlock = zeros(nbFrames, nbRows, nbCols); % motion fields
angleMapX = zeros(nbFrames, nbRows, nbCols); % motion field angles in x and y directions
angleMapY = zeros(nbFrames, nbRows, nbCols); % if 0, no motion was found
vDefined = logical(zeros(nbFrames, nbRows, nbCols));

singVal3 = zeros(nbFrames, nbRows, nbCols); % if 0, no motion was found
singVal2 = zeros(nbFrames, nbRows, nbCols); % if 0, no motion was found
singVal1 = zeros(nbFrames, nbRows, nbCols); % if 0, no motion was found

d = [-1, 8, 0, -8, 1]/12; % derivative template
mask = [0.0625,0.25,0.375,0.25,0.0625]; % weight template, sums to 1

% Apply spatio-temporal gaussian filter
if 0 % larger smoothing
   gaussF = (exp(-[-5:5].^2/(2*1.5*1.5))/sqrt(2*pi))'; % gaussian with sigma 1.5
else % smaller filter so there are some pixels left
   gaussF = (exp(-[-2:2].^2/(2*1.0*1.0))/sqrt(2*pi))'; % gaussian with sigma 1
end

gaussF = gaussF/sum(gaussF); % normalizing

if 1
	aBlock = filter3D(aBlock, gaussF);
	bBlock = filter3D(bBlock, gaussF);
	cBlock = filter3D(cBlock, gaussF);
end
at = filter1D(aBlock,d,1); 
ay = filter1D(aBlock,d,2);
ax = filter1D(aBlock,d,3);

bt = filter1D(bBlock,d,1); 
by = filter1D(bBlock,d,2);
bx = filter1D(bBlock,d,3);

ct = filter1D(cBlock,d,1); 
cy = filter1D(cBlock,d,2);
cx = filter1D(cBlock,d,3);


ItIt = at .* at + bt .* bt + ct .* ct;
IxIt = at .* ax + bt .* bx + ct .* cx;
IyIt = at .* ay + bt .* by + ct .* cy;
IxIx = ax .* ax + ax .* ax + cx .* cx;
IxIy = ax .* ay + bx .* by + cx .* cy;
IyIy = ay .* ay + by .* by + cy .* cy;


mask = [0.0625,0.25,0.375,0.25,0.0625];

vDefined = logical(zeros(nbFrames, nbRows, nbCols));
gaussF = (exp(-[-2:2].^2/(2*1.0*1.0))/sqrt(2*pi))'; % gaussian with sigma 1
gaussF = gaussF/sum(gaussF); % normalizing
gauss2 = gaussF * gaussF';

for i =1:nbFrames
    disp(i);
    
    fIxIx(:,:) = IxIx(i,:,:); fIyIy(:,:) = IyIy(i,:,:); fIxIy(:,:) = IxIy(i,:,:);
    fIxIt(:,:) = IxIt(i,:,:); fIyIt(:,:) = IyIt(i,:,:); fItIt = squeeze(ItIt(i,:,:));
    fIxIx=conv2(mask, mask, fIxIx, 'same');	
    fIyIy=conv2(mask, mask, fIyIy, 'same');	
    fIxIy=conv2(mask, mask, fIxIy, 'same');	
    fIxIt=conv2(mask, mask, fIxIt, 'same');	
    fIyIt=conv2(mask, mask, fIyIt, 'same');	
    fItIt=conv2(mask, mask, fItIt, 'same');	

    for j = 1:nbRows
	    for k = 1:nbCols
		    mat = [fItIt(j,k), fIxIt(j,k), fIyIt(j,k);...
		           fIxIt(j,k), fIxIx(j,k), fIxIy(j,k);
		           fIyIt(j,k), fIxIy(j,k), fIyIy(j,k)];

		    [UU,SS,VV] = svd(mat,0); % find best normal to set of gradient vectors
           singVal3(i,j,k) = SS(3,3)/SS(1,1);
%             singVal3(i,j,k) = SS(3,3);
           singVal2(i,j,k) = SS(2,2)/SS(1,1);
%             singVal2(i,j,k) = SS(2,2);
           singVal1(i,j,k) = SS(1,1);
%            SS(3,3)
            normal = VV(:,3); % 3D vect of length 1
            vDefined(i,j,k) = 1;
%              maxSingVal = 0.1;
            maxSingVal = 1;
            if 1
%                 vDefined(i,j,k) = SS(3,3) < maxSingVal; % larger is easier
                vDefined(i,j,k) = singVal3(i,j,k) < 2; % larger is easier
            else
                if SS(3,3) > maxSingVal
                    initNormal = normal;
                    [normal, isConverged] = reduceSingVal(mat, initNormal, maxSingVal);
                    vDefined(i,j,k) = isConverged;
                    t = normal(1); u = normal(2); v = normal(3);
                end
            end
           t = normal(1); u = normal(2); v = normal(3);
           if vDefined(i,j,k)
     		   angleMapX(i,j,k) = 90 - atan(u ./ t) * 180/pi; % from 1 to 179    
			   angleMapY(i,j,k) = 90 - atan(v ./ t) * 180/pi;
           else
               angleMapX(i,j,k) = 0;
               angleMapY(i,j,k) = 0;
           end; % if
	    end;
    end;
end;

% confidence = singVal2 ./(singVal3+0.001);

if 1 % check if necessary
    delta = 4;
	angleMapX = cropBlock(angleMapX, delta);
	angleMapY = cropBlock(angleMapY, delta);
% 	confidence = cropBlock(confidence, delta);
	singVal3 = cropBlock(singVal3, delta);
	singVal2 = cropBlock(singVal2, delta);
	singVal1 = cropBlock(singVal1, delta);
end

angleMapX = uint8(angleMapX);
slicePixelBlock(angleMapX);

angleMapY = uint8(angleMapY);

if 0
% 	max0 = max(confidence(:)); min0 = min(confidence(:));
% 	confVal = uint8(255 * (confidence - min0)/(max0-min0));
% 	slicePixelBlock(confVal);
	
	max3 = max(singVal3(:)); min3 = min(singVal3(:));
	singVal3 = uint8(255 * (singVal3 - min3)/(max3-min3));
	slicePixelBlock(singVal3);
	
	max2 = max(singVal2(:)); min2 = min(singVal2(:));
	singVal2 = uint8(255 * (singVal2 - min2)/(max2-min2));
	slicePixelBlock(singVal2);
	
	max1 = max(singVal1(:)); min1 = min(singVal1(:));
	singVal1 = uint8(255 * (singVal1 - min1)/(max1-min1));
	slicePixelBlock(singVal1);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	   
   


