function [angleMapX, angleMapY, vDefined] = makeSVDAngleMapsFromGrey(grey3D)

% Usage: [angleMapX, angleMapY, vDefinedBlock] = makeAngleMapsNoHoles(grey3D)
% Compute optical flow using Lucas-Kanade algorithm
% Here we don't return optical flow itself, only space-time angles. 
% We commented out uBlock and vBlock
% In makeAngleMaps

disp('Corrected alignments in derivative maps');
grey3D = double(grey3D);

% Apply spatio-temporal gaussian filter
if 0 % larger smoothing
	gaussF = (exp(-[-5:5].^2/(2*1.5*1.5))/sqrt(2*pi))'; % gaussian with sigma 1.5
	
	gaussF = gaussF/sum(gaussF); % normalizing
	
	grey3D = filter(gaussF, 1, grey3D, [], 1);
	grey3D(1:8,:,:) = [];% temporal dimension
	grey3D = filter(gaussF, 1, grey3D, [], 2); 
	grey3D(:,1:8,:) = [];% rows
	grey3D = filter(gaussF, 1, grey3D, [], 3); 
	grey3D(:,:,1:8) = [];% columns
	
else % smaller filter so there are some pixels left
	gaussF = (exp(-[-2:2].^2/(2*1.0*1.0))/sqrt(2*pi))'; % gaussian with sigma 1
	
	gaussF = gaussF/sum(gaussF); % normalizing
	
	grey3D = filter(gaussF, 1, grey3D, [], 1);
	grey3D(1:4,:,:) = [];% temporal dimension
	grey3D = filter(gaussF, 1, grey3D, [], 2);
	grey3D(:,1:4,:) = [];% rows
	grey3D = filter(gaussF, 1, grey3D, [], 3);
	grey3D(:,:,1:4) = [];% columns

end
% Compute spatio-temporal derivatives

[nbFrames, nbRows, nbCols] = size(grey3D);
% Compute spatio-temporal derivatives
d = [-1, 8, 0, -8, 1]/12;

% Filter displaces pixel blocks by 2 pixels away from edge along fitering dimension
% Also we have to keep images aligned and of same size
It = filter(d,1,grey3D,[],1); 
It(1:4,:,:) = []; % 2 pixels off to realign image and 2 off to trim bad pixels
grey3D(nbFrames-1:nbFrames,:,:) = [];
grey3D(1:2,:,:) = []; % to realign

Iy = filter(d,1,grey3D,[],2);
Iy(:,1:4,:) = [];
It(:,nbRows-1:nbRows,:) = [];
It(:,1:2,:) = [];
grey3D(:,nbRows-1:nbRows,:) = [];
grey3D(:,1:2,:) = [];

Ix = filter(d,1,grey3D,[],3);
Ix(:,:,1:4) = [];
It(:,:,nbCols-1:nbCols) = [];
It(:,:,1:2) = [];
Iy(:,:,nbCols-1:nbCols) = [];
Iy(:,:,1:2) = [];

[nbFrames, nbRows, nbCols] = size(It);

angleMapX = zeros(nbFrames, nbRows, nbCols); % motion fields
angleMapY = zeros(nbFrames, nbRows, nbCols); % if 0, no motion was found

IxIx=Ix.*Ix;
IyIy=Iy.*Iy;
IxIy=Ix.*Iy;
IxIt=Ix.*It;
IyIt=Iy.*It;
ItIt=It.*It;



mask = [0.0625,0.25,0.375,0.25,0.0625];

vDefined = logical(zeros(nbFrames, nbRows, nbCols));
gaussF = (exp(-[-2:2].^2/(2*1.0*1.0))/sqrt(2*pi))'; % gaussian with sigma 1
gaussF = gaussF/sum(gaussF); % normalizing
gauss2 = gaussF * gaussF';

for i =1:nbFrames
    disp(i);
    
    fIxIx(:,:) = IxIx(i,:,:); fIyIy(:,:) = IyIy(i,:,:); fIxIy(:,:) = IxIy(i,:,:);
    fIxIt(:,:) = IxIt(i,:,:); fIyIt(:,:) = IyIt(i,:,:); fItIt = squeeze(ItIt(i,:,:));
    fIxIx=conv2(mask, mask, fIxIx, 'same');	
    fIyIy=conv2(mask, mask, fIyIy, 'same');	
    fIxIy=conv2(mask, mask, fIxIy, 'same');	
    fIxIt=conv2(mask, mask, fIxIt, 'same');	
    fIyIt=conv2(mask, mask, fIyIt, 'same');	
    fItIt=conv2(mask, mask, fItIt, 'same');	

    for j = 1:nbRows
	    for k = 1:nbCols
		    mat = [fItIt(j,k), fIxIt(j,k), fIyIt(j,k);...
		           fIxIt(j,k), fIxIx(j,k), fIxIy(j,k);
		           fIyIt(j,k), fIxIy(j,k), fIyIy(j,k)];

		    [UU,SS,VV] = svd(mat,0); % find best normal to set of gradient vectors
           singVal3(i,j,k) = SS(3,3)/SS(1,1);
%             singVal3(i,j,k) = SS(3,3);
           singVal2(i,j,k) = SS(2,2)/SS(1,1);
%             singVal2(i,j,k) = SS(2,2);
           singVal1(i,j,k) = SS(1,1);
%            SS(3,3)
            normal = VV(:,3); % 3D vect of length 1
            vDefined(i,j,k) = 1;
%              maxSingVal = 0.1;
            maxSingVal = 1;
            if 1
%                 vDefined(i,j,k) = SS(3,3) < maxSingVal; % larger is easier
%                 vDefined(i,j,k) = singVal3(i,j,k) < 0.01; % larger is easier
                vDefined(i,j,k) = singVal3(i,j,k) < 1; % larger is easier
            else
                if SS(3,3) > maxSingVal
                    initNormal = normal;
                    [normal, isConverged] = reduceSingVal(mat, initNormal, maxSingVal);
                    vDefined(i,j,k) = isConverged;
                    t = normal(1); u = normal(2); v = normal(3);
                end
            end
           t = normal(1); u = normal(2); v = normal(3);
           if vDefined(i,j,k)
     		   angleMapX(i,j,k) = 90 - atan(u ./ t) * 180/pi; % from 1 to 179    
			   angleMapY(i,j,k) = 90 - atan(v ./ t) * 180/pi;
           else
               angleMapX(i,j,k) = 0;
               angleMapY(i,j,k) = 0;
           end; % if
	    end;
    end;
end;

confidence = singVal2 ./(singVal3+0.001);

if 0 % check if necessary
    delta = 2;
	angleMapX = cropBlock(angleMapX, delta);
	confidence = cropBlock(confidence, delta);
	singVal3 = cropBlock(singVal3, delta);
	singVal2 = cropBlock(singVal2, delta);
	singVal1 = cropBlock(singVal1, delta);
end

angleMapX = uint8(angleMapX(:,3:nbRows-2,3:nbCols-2));
angleMapY = uint8(angleMapY(:,3:nbRows-2,3:nbCols-2));
slicePixelBlock(angleMapX);


if 0
	max1 = max(singVal1(:)); min1 = min(singVal1(:));
	singVal1 = uint8(255 * (singVal1 - min1)/(max1-min1));
	slicePixelBlock(singVal1);

	max2 = max(singVal2(:)); min2 = min(singVal2(:));
	singVal2 = uint8(255 * (singVal2 - min2)/(max2-min2));
	slicePixelBlock(singVal2);
	
	max3 = max(singVal3(:)); min3 = min(singVal3(:));
	singVal3 = uint8(255 * (singVal3 - min3)/(max3-min3));
	slicePixelBlock(singVal3);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

