function [angleMapX, angleMapY, vDefined] = makeSVDAngleMapsFromGrey2(grey3D)

% Usage: [angleMapX, angleMapY, vDefinedBlock] = makeSVDAngleMapsFromGrey2(grey3D)
% Here we don't return optical flow itself, only space-time angles. 

[nbFrames, nbRows, nbCols] = size(grey3D);
uBlock = zeros(nbFrames, nbRows, nbCols); % motion fields
vBlock = zeros(nbFrames, nbRows, nbCols); % motion fields
angleMapX = zeros(nbFrames, nbRows, nbCols); % motion field angles in x and y directions
angleMapY = zeros(nbFrames, nbRows, nbCols); % if 0, no motion was found
vDefined = logical(zeros(nbFrames, nbRows, nbCols));

singVal3 = zeros(nbFrames, nbRows, nbCols); % if 0, no motion was found
singVal2 = zeros(nbFrames, nbRows, nbCols); % if 0, no motion was found
singVal1 = zeros(nbFrames, nbRows, nbCols); % if 0, no motion was found

d = [-1, 8, 0, -8, 1]/12; % derivative template
mask = [0.0625,0.25,0.375,0.25,0.0625]; % weight template, sums to 1

% Apply spatio-temporal gaussian filter
if 0 % larger smoothing
   gaussF = (exp(-[-5:5].^2/(2*1.5*1.5))/sqrt(2*pi))'; % gaussian with sigma 1.5
else % smaller filter so there are some pixels left
   gaussF = (exp(-[-2:2].^2/(2*1.0*1.0))/sqrt(2*pi))'; % gaussian with sigma 1
end

% gaussF = gaussF/sum(gaussF); % normalizing

if 1
	grey3D = filter3D(grey3D, gaussF);
end

% Compute spatio-temporal derivatives

% Compute spatio-temporal derivatives
d = [-1, 8, 0, -8, 1]/12;
It = filter1D(grey3D,d,1); 
Iy = filter1D(grey3D,d,2);
Ix = filter1D(grey3D,d,3);

IxIx=Ix.*Ix;
IyIy=Iy.*Iy;
IxIy=Ix.*Iy;
IxIt=Ix.*It;
IyIt=Iy.*It;
ItIt=It.*It;



mask = [0.0625,0.25,0.375,0.25,0.0625];

vDefined = logical(zeros(nbFrames, nbRows, nbCols));
gaussF = (exp(-[-2:2].^2/(2*1.0*1.0))/sqrt(2*pi))'; % gaussian with sigma 1
gaussF = gaussF/sum(gaussF); % normalizing
gauss2 = gaussF * gaussF';

for i =1:nbFrames
    disp(i);
    
    fIxIx(:,:) = IxIx(i,:,:); fIyIy(:,:) = IyIy(i,:,:); fIxIy(:,:) = IxIy(i,:,:);
    fIxIt(:,:) = IxIt(i,:,:); fIyIt(:,:) = IyIt(i,:,:); fItIt = squeeze(ItIt(i,:,:));
    fIxIx=conv2(mask, mask, fIxIx, 'same');	
    fIyIy=conv2(mask, mask, fIyIy, 'same');	
    fIxIy=conv2(mask, mask, fIxIy, 'same');	
    fIxIt=conv2(mask, mask, fIxIt, 'same');	
    fIyIt=conv2(mask, mask, fIyIt, 'same');	
    fItIt=conv2(mask, mask, fItIt, 'same');	

    for j = 1:nbRows
	    for k = 1:nbCols
		    mat = [fItIt(j,k), fIxIt(j,k), fIyIt(j,k);...
		           fIxIt(j,k), fIxIx(j,k), fIxIy(j,k);
		           fIyIt(j,k), fIxIy(j,k), fIyIy(j,k)];

		    [UU,SS,VV] = svd(mat,0); % find best normal to set of gradient vectors
%            singVal3(i,j,k) = SS(3,3)/SS(1,1);
            singVal3(i,j,k) = SS(3,3);
%            singVal2(i,j,k) = SS(2,2)/SS(1,1);
            singVal2(i,j,k) = SS(2,2);
            singVal1(i,j,k) = SS(1,1);
%            SS(3,3)
            normal = VV(:,3); % 3D vect of length 1
            vDefined(i,j,k) = 1;
%              maxSingVal = 0.1;
            maxSingVal = 1;
            if 1
%                 vDefined(i,j,k) = SS(3,3) < maxSingVal; % larger is easier
%                 vDefined(i,j,k) = singVal3(i,j,k) < 0.01; % larger is easier
%                 vDefined(i,j,k) = singVal3(i,j,k) < 1; % larger is easier
 % we want large singVal2 (no aperture problem) and small singVal3 (no overdetermined field
                  vDefined(i,j,k) = singVal2(i,j,k) > 1 & singVal3(i,j,k) < 1; 
            else
                if SS(3,3) > maxSingVal
                    initNormal = normal;
                    [normal, isConverged] = reduceSingVal(mat, initNormal, maxSingVal);
                    vDefined(i,j,k) = isConverged;
                end
            end
           t = normal(1); u = normal(2); v = normal(3);
           if vDefined(i,j,k)
     		   angleMapX(i,j,k) = 90 - atan(u ./ t) * 180/pi; % from 1 to 179    
		   angleMapY(i,j,k) = 90 - atan(v ./ t) * 180/pi;
           else
               angleMapX(i,j,k) = 0;
               angleMapY(i,j,k) = 0;
           end; % if
	    end;
    end;
end;

% confidence = singVal2 ./(singVal3+0.001);


angleMapX = uint8(angleMapX(5:nbFrames-4,5:nbRows-4,5:nbCols-4));
angleMapY = uint8(angleMapY(5:nbFrames-4,5:nbRows-4,5:nbCols-4));
vDefined = vDefined(5:nbFrames-4,5:nbRows-4,5:nbCols-4);

slicePixelBlock(angleMapX);

if 0
	max1 = max(singVal1(:)); min1 = min(singVal1(:));
    disp(['min and max for singVal1 ', num2str(min1), ', ', num2str(max1)]);
	singVal1 = uint8(255 * (singVal1 - min1)/(max1-min1));
	slicePixelBlock(singVal1);

	max2 = max(singVal2(:)); min2 = min(singVal2(:));
    disp(['min and max for singVal2 ', num2str(min2), ', ', num2str(max2)]);
	singVal2 = uint8(255 * (singVal2 - min2)/(max2-min2));
	slicePixelBlock(singVal2);
	
	max3 = max(singVal3(:)); min3 = min(singVal3(:));
    disp(['min and max for singVal3 ', num2str(min3), ', ', num2str(max3)]);
	singVal3 = uint8(255 * (singVal3 - min3)/(max3-min3));
	slicePixelBlock(singVal3);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

