function grey = makeTestSequence
%
% Make a moving difference of gaussians
% with same 45 degree motion as flower garden tree
% i.e. around 15 pixels over 15 frames

nbFrames = 16;
nbRows = 60;
nbCols = 88;
coeff = 1/sqrt(2*pi);
twoSigmaSquared1 = 2 * 12.0 * 10.0;
twoSigmaSquared2 = 2 * 8.0 * 6.0;

grey = zeros(nbFrames, nbRows, nbCols);

rowCounts = ones(nbCols,1) * [1:nbRows];
rowCounts = rowCounts';
colCounts = ones(nbRows,1) * [1:nbCols];
centerRow = nbRows/2;
squaredYFromCenter = (rowCounts -centerRow) .* (rowCounts -centerRow);
for i = 1:nbFrames
    disp(i);
    shift = i;
    centerCol = nbCols/2 - nbFrames/2 + shift;
    squareXFromCenter = (colCounts -centerCol) .* (colCounts -centerCol);
    squaredDist = squareXFromCenter + squaredYFromCenter;
    grey(i, :,:) = coeff*(exp(-squaredDist/twoSigmaSquared1) - exp(-squaredDist/twoSigmaSquared2));
end;

maxGrey = max(grey(:));
minGrey = min(grey(:));

grey = 256 * (grey - minGrey)/(maxGrey - minGrey);

grey = uint8(grey);
