function r = nonMaximalSuppress(gradX, gradY)
%      Syntax: r = nonMaximalSuppress(gradX, gradY)
%
%         Arg: gradX(Y)    - local 3x3 gradient matrix.
%              r           - boolean result.
%                              1: is local maxima
%                              0: is NOT local maxima
%
% Description: Determine if the center of "local" is the max gradient
%
% Author: Haiying
% Date  : Oct 3, 2000

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Check arguments.

msg = nargchk(2, 2, nargin);
if (~isempty(msg))
  error(strcat('ERROR:', msg));
end

clear msg;

%======================================================================
%= For each pixel (except those on the image boundary), check if 
%= the gradient magnitude is a local (3x3 neighbourhood) maxima in 
%= the gradient orientation:

r = 0;

y = 2;
x = 2;

dx = gradX(y, x);
dy = gradY(y, x);

mag = sqrt(gradX .* gradX + gradY .* gradY);

if (dx ~= 0 | dy ~= 0)
  if (abs(dy) > abs(dx))		    
    ux = abs(dx) / abs(dy);
    uy = 1;
    
    gb = mag(y - 1, x);
    gd = mag(y + 1, x);
    
    if (sign(dx) ~= sign(dy))
      ga = mag(y - 1, x - 1);
      gc = mag(y + 1, x + 1);
    else
      ga = mag(y - 1, x + 1);
      gc = mag(y + 1, x - 1);
    end
  else
    ux = abs(dy) / abs(dx);
    uy = 1;
    
    gb = mag(y, x + 1);
    gd = mag(y, x - 1);
    
    if (sign(dx) ~= sign(dy))
      ga = mag(y + 1, x + 1);
      gc = mag(y - 1, x - 1);
    else
      ga = mag(y - 1, x + 1);
      gc = mag(y + 1, x - 1);
    end
  end
  
  g1 = (ux * ga) + (uy - ux) * gb;
  g2 = (ux * gc) + (uy - ux) * gd;
  g  = sqrt(dx * dx + dy * dy);
  
  if (g >= g1 & g >= g2)
    r = 1;
  else
    r = 0;
  end
end
