function luv = rgb2Luv2(im)
% takes an image in RGB coords, 
% and returns an image
% in Luv coords

[nbRows, nbCols, nbChannels] = size(im);
im = double(im);
nbPixels = nbRows * nbCols;
pixList = shiftdim(im, 2);
pixList = double(reshape(pixList, [3, nbPixels]));
pix_nz = (sum(pixList, 1) > 0);
pix_z = not(pix_nz);
pixList = max(pixList, [pix_z; pix_z; pix_z]);	% replaces all zero pixels with [1;1;1]

pixList = pixList/255;

xyzMat = [0.412453, 0.35758 , 0.180423;...
		  0.212671, 0.71516 , 0.072169;...
		  0.019334, 0.119193, 0.950227];

xyz = xyzMat*pixList;		% now get a vector in XYZ coords

whitePt = xyzMat * [1;1;1];
x0 = whitePt(1); y0 = whitePt(2); z0 = whitePt(3);

u0 = 4*x0/(x0+15*y0+3*z0);
v0 = 9*y0/(x0+15*y0+3*z0);

luv = zeros(size(pixList));
luv(1, :) = 116 * (xyz(2, :)/y0).^(0.333)-16;
u = 4*xyz(1, :) ./ (xyz(1, :)+15*xyz(2, :)+3*xyz(3, :));
v = 9*xyz(2, :) ./ (xyz(1, :)+15*xyz(2, :)+3*xyz(3, :));
luv(2, :) = 13*luv(1, :).*(u-u0);
luv(3, :) = 13*luv(1, :).*(v-v0);
luv = reshape(luv, [3, nbRows, nbCols]);
luv = shiftdim(luv, 1);
