/*=================================================================
 *
 * fastCompleteMergeDups.c  
 *
 * Usage: [newFeatureList, newWeights, newTrace] = fastCompleteMergeDups(featureList, sortedKeys, inverseTrace)
 * Inputs:
 * 1. featureList, list of feature vectors, matrix N X M for N vectors with M components
 * 2. sortedKeys, list of sorted keys, see fastMergeDups.m
 * 3. inverseTrace, index mapping showing where each of original indices went after sorting
 * sortedKeys = keys(inverseTrace)
 * Outputs:
 * 1. newFeatureList: List of feature vectors in which duplicates have been removed
 * 2. newWeights: number of duplicates corresponding to each vector of newFeatureList
 * 3. newTrace: Index mapping from featureList to newFeatureList
 *
 * This is a MEX-file for MATLAB.  
 * Copyright 2001 Daniel DeMenthon & University of Maryland 
 *
 *=================================================================*/

#include <math.h>
#include "mex.h"

//=========================================================================

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  int i, j, vectCoordIndex, newVectCoordIndex, index, newIndex, originalIndex;
  int nbVects, nbFeatures;
  double *featureList, *sortedKeys, *inverseTrace, *newFeatureList, *newWeights, *newTrace;
  double vectCoord, key, newKey;

  nbVects = mxGetM(prhs[0]); /* nb of pts in shiftList */
  nbFeatures = mxGetN(prhs[0]); /* nb of feature components */

  featureList = mxGetPr(prhs[0]);
  sortedKeys = mxGetPr(prhs[1]);
  inverseTrace = mxGetPr(prhs[2]);

  plhs[0] = mxCreateDoubleMatrix(nbVects, nbFeatures, mxREAL); // max size; reduce size in Matlab
  newFeatureList = mxGetPr(plhs[0]);

  plhs[1] = mxCreateDoubleMatrix(nbVects, 1, mxREAL); // max size; reduce size in Matlab
  newWeights = mxGetPr(plhs[1]);

  plhs[2] = mxCreateDoubleMatrix(nbVects, 1, mxREAL); // actual size
  newTrace = mxGetPr(plhs[2]);

  key = -1; // so that newKey==key is false at first pass, since newKey is positive
  index = -1; // so that we start with index 0
  
  for(i=0;i<nbVects;i++){
//  mexPrintf("i %d \n", i);
    newKey = sortedKeys[i];
//  mexPrintf("key, newKey %12.0f %12.0f\n", key, newKey);
    originalIndex = inverseTrace[i] - 1; // -1 since arrays are zero-based
    if(newKey == key){
      newIndex = index;
      newWeights[newIndex]++;
    }
    else{
      newIndex = index + 1;
      newWeights[newIndex] = 1;
      for(j=0;j<nbFeatures;j++){// copy featureList[originalIndex] to newFeatureList[newIndex]
	vectCoordIndex = j * nbVects + originalIndex; // matrices are stored by column
	vectCoord = *(featureList + vectCoordIndex);
	newVectCoordIndex = j * nbVects + newIndex;
	*(newFeatureList + newVectCoordIndex) = vectCoord;
      }
    }// else
    newTrace[originalIndex] = newIndex + 1;// +1 since Matlab arrays are one-based
//  mexPrintf("newTrace %f\n", newTrace[originalIndex]);
    key = newKey;
    index = newIndex;
  }
}  
     

  
