/*=================================================================
 *
 * fastFindKeys.c  
 *
 * Usage: keys = fastFindKeys(featureList, quantizError)
 * Inputs:
 * 1. featureList, list of feature vectors, matrix N X M for N vectors with M components
 * Components have been normalized in range 0 to 1
 * 2. quantizError quantization error; example: quantizError = 1/512 for normalized components
  *
 * Outputs:
 * keys: List of numbers, one for each feature vector, such that keys are identical if vectors are
 * and different if vectors are
 *
 * This is a MEX-file for MATLAB.  
 * Copyright 2001 Daniel DeMenthon & University of Maryland 
 *
 *=================================================================*/

#include <math.h>
#include "mex.h"

//=========================================================================

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )     
{
  int i, j;
  int nbVects, nbFeatures;
  double *featureList, *quantizError;
  double *keys;

  double newKey, tens, vectCoord;
  long number, vectCoordIndex, digit, newNumber;

  nbVects = mxGetM(prhs[0]); /* nb of pts in shiftList */
  nbFeatures = mxGetN(prhs[0]); /* nb of feature components */

  featureList = mxGetPr(prhs[0]);
  quantizError = mxGetPr(prhs[1]); // Note: this is a pointer

  plhs[0] = mxCreateDoubleMatrix(nbVects, 1, mxREAL); // max size; reduce size in Matlab
  keys = mxGetPr(plhs[0]);

  for(i=0;i<nbVects;i++){
    newKey = 0;
    tens = 1;
    for(j=0;j<nbFeatures;j++){
      vectCoordIndex = j * nbVects + i; // matrices are stored by column
      vectCoord = *(featureList + vectCoordIndex);
      number = (long)(vectCoord/quantizError[0] + 0.5); // rounded coordinate

      while(1){
	newNumber = (long)(number/10.0);
	digit = number - 10 * newNumber;
	newKey = newKey + tens * digit;
	number = newNumber;
	tens = 10.0 * tens;
	if(number == 0){
	  break;
	}// end if
      }// end while
    }// end for j
    keys[i] = newKey;
  }// end for i
} 
      
      

  
