function [newFeatureList, newWeights, newTrace] = fastMergeDups(featureList, quantizError)
%
% Usage: [newFeatureList, newWeights, newTrace] = fastMergeDups(featureList, quantizError)
%
% Replace components of each feature vector by a single number, called a key
% Sort them, check for duplicates along list
% Increment index when no duplicate
% Use inverse trace obtained at sorting to build trace
% This is a new sorting version that should work better than tree approach 
% when there are a lot of duplicates

% tic
[nbVects, nbFeatures] = size(featureList); 

disp('~~~~~~~~~~~~~~~~~~~~~~ Version with component swaps ~~~~~~~~~~~~~~~~~~~~~~~');

keys = zeros(nbVects,1);
newTrace = zeros(nbVects,1); % we trim these when we are done
newWeights = zeros(nbVects,1);
newFeatureList = zeros(nbVects,nbFeatures);

% disp('1. Make list of keys');
if 1
    keys = fastFindKeys(featureList, quantizError); % call to mex function
else % slow; left here as documentation   
	roundedFeatureList = round(featureList/quantizError);
	
	for i = 1:nbVects
        newKey = 0;
        tens = 1;
		for j = 1:nbFeatures
            number = roundedFeatureList(i,j);
            while 1
                newNumber = floor(number/10.0);
                digit = number - 10 * newNumber;
                newKey = newKey + tens * digit;
                number = newNumber;
                tens = 10.0 * tens;
                if number == 0
                    break;
                end; % if
            end; % while
		end; % for j
		keys(i) = newKey; % double precision number
	end; % for i
end; % if 1

% disp('2. Sort keys');
% inverse trace lists index of old list that contribute to new
[sortedKeys, inverseTrace] = sort(keys); % sortedKeys = keys(inverseTrace)

% disp('3. Compute weights and trace');
if 1
    [newFeatureList, newWeights, newTrace] = fastCompleteMergeDups(featureList, sortedKeys, inverseTrace);

else % slow version, kept as documentation
    
    key = -1;
    index = 0;
	for i = 1:nbVects
		newKey = sortedKeys(i);
		originalIndex = inverseTrace(i);
		if newKey == key
			newIndex = index;
			newWeights(newIndex) = newWeights(newIndex) + 1;
		else
			newIndex = index + 1;
			newWeights(newIndex) = 1;
			newFeatureList(newIndex,:) = featureList(originalIndex,:);
		end; % if
		newTrace(originalIndex) = newIndex;
		key = newKey;
		index = newIndex;
	end;    
end; % if 0

nbNewVects = max(newTrace);
newFeatureList = newFeatureList(1:nbNewVects,:);
newWeights = newWeights(1:nbNewVects);

% newWeights = findWeights(newTrace);

% toc
		 
	
	
