function clusterDensities = findClusterDensities(shiftList, featureVects, weights, treeData, radius);
%
% Usage: clusterDensities = findClusterDensities(shiftList, featureVects, weights, treeData, radius);
%
% Find densities of points within radius range of clusters 
% using the weight of each point 
% This the same weightsInSphere used in fastShiftTreeFeatures
% but we use the term densities instead of weights to avoid confusion
% with weights of all the pixels that converged to clusters
% We could have returned weightInSphere from fastShiftTreeFeatures
% This postprocessing approach avoids keeping tab of densities
% until the shiftList vectors have stopped shifting

[nbPts] = size(shiftList, 1);
clusterDensities = zeros(nbPts, 1);

[counts, neighbors] = range_search(featureVects, treeData, shiftList, radius, 0); % list of neighbors, excludes self-match

neighborIndices = neighbors(:,1); % pass only the part of neighbors that is useful (distances are not)

for i = 1:nbPts
    indicesInRange = [i, neighborIndices{i}]; % should have at least self in neighbor list
    weightsInSphere = weights(indicesInRange);
    totalWeight = sum(weightsInSphere, 1);
    clusterDensities(i) = totalWeight;
end;


