function bestTrace = findOptimalClustering2(allTraces)
%
% Usage: bestTrace = findOptimalClustering2(allTraces)
%
% Return a trace in which each region corresponds to the longest living clusters when radius is increased
% and the corresponding clusters (later)

% 1. Check if there was a grouping between parent trace and child trace
% if there is, relabel new cluster with a new global label


[nbPixels, nbTraces] = size(allTraces);

bestTrace = allTraces(:,1); % this sets zeros for pixels with no clustering
nbBestClusters = max(bestTrace);
bestAges = ones(nbBestClusters,1); % ages of best clusters
ages = ones(nbBestClusters,1); % ages of initial clusters

allocatedLabels = nbBestClusters; % every cluster has distinct label in first trace
isBestTraceUpdated = zeros(nbBestClusters);

for i = 2:nbTraces 
   i
   childTrace = allTraces(:,i-1);
   parentTrace = allTraces(:,i);
   nbParentClusters = max(parentTrace); % note: this would not work if we relabeled all traces
   for j = 1:nbParentClusters
		parentIndices = find(parentTrace == j); % pixels that have label j in parents
		childRegionLabels = childTrace(parentIndices); % find cluster labels in child for those pixels
        maxLabel = max(childRegionLabels);
        minLabel = min(childRegionLabels);
		if maxLabel == minLabel % then single label in child, no grouping, age of cluster is increased
			clusterLabel = maxLabel;
    		allTraces(parentIndices, i) = clusterLabel; % replace parent label with index of child, so we can track age
            ages(clusterLabel) = ages(clusterLabel) + 1;
			if isBestTraceUpdated(clusterLabel)
				bestAges(clusterLabel) = bestAges(clusterLabel) + 1;            
            else % bestTrace has not be updated yet for this cluster
	            bestLocalLabels = bestTrace(parentIndices);
	            bestLocalAges = bestAges(bestLocalLabels);
	            oldestAge = max(bestLocalAges); % find oldest cluster overlapped by maxLabel cluster
            	if ages(clusterLabel) > oldestAge
%             	if ages(clusterLabel) >= oldestAge - 5
%             	if ages(clusterLabel) > sqrt(oldestAge)
            		bestTrace(parentIndices) = clusterLabel; % place clusterLabel in pixels of bestTrace
                    bestAges(clusterLabel) = ages(clusterLabel);
            		isBestTraceUpdated(maxLabel) = 1;
            	end;
            end;
        else  % a grouping occurred
            allocatedLabels = allocatedLabels + 1;
            allTraces(parentIndices, i) = allocatedLabels; % a new label for the grouping
            ages(allocatedLabels) = 1; % no  point checking ages of bestTraces
            isBestTraceUpdated(allocatedLabels) = 0;
        end;
   end; % for j
end; % for i

disp('Reassign bestTrace labels');
% Labels in bestTrace have missing elements in label sequence;
% replace current labels using a continuous sequence from 1 to nb of labels

maxLabel = max(bestTrace);
labelMapping = zeros(maxLabel);
availableLabel = 0;


for i = 1:nbPixels
    label = bestTrace(i);
    if label ~= 0
        newLabel = labelMapping(label);
        if newLabel == 0 % was not reassigned yet
            availableLabel = availableLabel + 1;
            labelMapping(label) = availableLabel;
            bestTrace(i) = availableLabel;
        else
            bestTrace(i) = newLabel;
        end
    end
end
