function normalizedFeatures = normalizeToRanges(featureList, ranges, importances)
%
% Usage: [normalizedFeatures] = normalizeToRanges(featureList, ranges, importances)
% map feature coordinates using given ranges
% so that results could fall beyond range [0,1]
% also reduce range according to importances, which are numbers between 0 and 1
%

[nbPts, nbFeatures] = size(featureList);
normalizedFeatures = featureList; % copy zero terms

lo = 1; hi = 2;

for i = 1:nbFeatures
    compMin = ranges(i,lo);
    compMax = ranges(i,hi);
    delta = compMax-compMin;
	if delta ~= 0 % else copy above is enough
	    normalizedFeatures(:,i) = importances(i) * (featureList(:,i) - compMin)/delta;
	else % delta = 0, set within hypercube 1
	    normalizedFeatures(:,i) = featureList(:,i)/compMax;
	end;
end;
