function [newFeatureList, newWeights, newTrace] = fastMergeDups(featureList, quantizError)
%
% Usage: [newFeatureList, newWeights, newTrace] = fastMergeDups(featureList, quantizError)
%
% 1. Bring together close feature vectors by dividing by quantization error and rounding
% 2. Find close rounded vectors by sorting them using Matlab sortrows
% 3. Make differences between consecutive rounded vectors
% 4. Find list of consecutive zeros in results and make a new list of feature vectors with new labels
% and compute map from old labels to new labels

% tic
[nbVects, nbFeatures] = size(featureList); 


keys = zeros(nbVects,1);
newTrace = zeros(nbVects,1); % we trim these when we are done
newWeights = zeros(nbVects,1);
newFeatureList = zeros(nbVects,nbFeatures);

if 0 % previous version
% Replace components of each feature vector by a single number, called a key
% Sort them, check for duplicates along list
% Increment index when no duplicate
% Use inverse trace obtained at sorting to build trace
% This is a new sorting version that should work better than tree approach 
% when there are a lot of duplicates
    % disp('1. Make list of keys');
    keys = fastFindKeys(featureList, quantizError); % call to mex function
    [sortedKeys, inverseTrace] = sort(keys); % sortedKeys = keys(inverseTrace)
    [newFeatureList, newWeights, newTrace] = fastCompleteMergeDups(featureList, sortedKeys, inverseTrace);
else % revised version,    
	disp('~~~~~~~~~~~~~~~~~~~~~~ Version with sortrows ~~~~~~~~~~~~~~~~~~~~~~~');
	roundedFeatureList = round(featureList/quantizError);
	[sortedFeatureList, inverseTrace] = sortrows(roundedFeatureList);
	shiftedFeatureList = sortedFeatureList([end 1:end-1],:);
	diffFeatures = sortedFeatureList - shiftedFeatureList;
   	sqDiffFeatures = diffFeatures .* diffFeatures;
	vectorDiffs = sum(sqDiffFeatures,2); % zero at i wherever vector(i-1) = vector(i) 

    % disp('3. Compute weights and trace');    
    vectorDiffs(1) = 1; % so that index of first element is set to 1 below
    index = 0;
	for i = 1:nbVects
		originalIndex = inverseTrace(i);
		if vectorDiffs(i) == 0 % feature vector has not changed compared to previous one
			newIndex = index;
			newWeights(newIndex) = newWeights(newIndex) + 1;
		else
			newIndex = index + 1;
			newWeights(newIndex) = 1;
			newFeatureList(newIndex,:) = featureList(originalIndex,:);
		end; % if
		newTrace(originalIndex) = newIndex;
		index = newIndex;
	end   
end

nbNewVects = max(newTrace);
newFeatureList = newFeatureList(1:nbNewVects,:);
newWeights = newWeights(1:nbNewVects);

% newWeights = findWeights(newTrace);

% toc
		 
	
	
