function [newShiftList, newShiftFlags] = fastShiftTreeFeatures(shiftList, featureVects, weights, treeData, radius, quantizError, doGaussian)
%
% Usage: [movedShiftList, newShiftFlags] = fastShiftTreeFeatures(shiftList, featureVects, weights, treeData, radius, quantizError)
%
% 
% Use ATRIA tree to find background points inside spheres around vectors of shiftList
% This version calls a loop written in C

newShiftList = shiftList;

[nbPts, nbFeatures] = size(shiftList);

disp('range_search');
% shiftList is different from background so exclude self-match
[counts, neighbors] = range_search(featureVects, treeData, shiftList, radius, 0); % list of neighbors
neighborIndices = neighbors(:,1); % pass only the part of neighbors that is useful (distances are not)

if 1
	disp('fast centroids');
    if doGaussian
        disp('%%% Gaussian weighting %%%');
     	sigma = 0.3 * radius; % so that the Gaussian is down to zero at the borders of the range search
        distances = neighbors(:,2);
		newShiftList = fastGaussCentroids(counts, neighborIndices, distances, featureVects, weights, shiftList, sigma); % call to MEX-file
    else   
        disp('+++ Flat weighting +++');
	    newShiftList = fastCentroids(counts, neighborIndices, featureVects, weights, shiftList); % call to MEX-file
    end;
	disp('fast findShiftFlags');
	newShiftFlags = findShiftFlags(shiftList, newShiftList, quantizError); % call to MEX-file

else % The slow version for fastCentroids, shown here as documentation to fastCentroids and findShiftFlags MEX-files
        
    disp('Slow centroids and findShiftFlags');
	newShiftFlags = zeros(nbPts,1);
	for i = 1:nbPts
	%  i
        nbPtsInSphere = counts(i);
        if nbPtsInSphere == 0
           centroid = shiftList(i,:);
           newShiftFlags(i) = 0;
       else
            indicesInRange = neighborIndices{i}; 
            ptsInSphere = featureVects(indicesInRange,:); % range_search returns indices of point set
            weightsInSphere = weights(indicesInRange);
            weightMat = weightsInSphere * ones(1,nbFeatures); % make weight matrix same size as pt matrix
            weightTimesCentroid = sum(weightMat .* ptsInSphere, 1);
            totalWeight = sum(weightsInSphere, 1);
            centroid = weightTimesCentroid / totalWeight;
            newShiftFlags(i) = any(abs(centroid - shiftList(i,:)) > quantizError); % flag vector that did move
       end; % if	
       newShiftList(i,:) = centroid;
	end; % for i
	
end; % if
