function [clusterCenters, clusterTrace] = findMaxNeighbors(shiftList, clusterDensities, radius)

% Usage: 
% [clusterList, clusterWeights, clusterTrace] = findMaxNeighbors(shiftList, clusterDensities, radius)
%
% Find subsets of all clusters that are within radius of at least one other
% and pick up the one with largest amount of neighbors 
% (rather than taking centroid as in previous versions)
% We also keep track of which pixels contributed to which group in clusterTrace
% It is used eventually to assign a cluster feature vector to each pixel of the shot image
% We use ATRIA tree instead of histograms or sparse matrices
% 10/09/01: clusterDensities are total weights of shiftList points within radius of clusters

tic
treeData = nn_prepare(shiftList); % building ATRIA tree
disp('range_search');
[counts, neighbors] = range_search(shiftList, treeData, shiftList, radius, 0); % list of neighbors
neighborIndices = neighbors(:,1);

if 1
    disp('Fast code for finding clusters with max density using mex-file');
   [clusterCenters, clusterTrace, nbClusters] = fastClusterMax(shiftList, clusterDensities, neighborIndices); 
    clusterCenters = clusterCenters(1:nbClusters,:);

else % left for documentation of algorithm and debugging
    disp('Slow Matlab code for finding cluster with max density among neighbors');
	[nbPts, nbFeatures] = size(shiftList);
	
	clusterTrace = zeros(nbPts,1);
	clusterCenters = zeros(nbPts, nbFeatures); % we prune afterwards
	
	usedPts = zeros(nbPts,1); % keep track of feature vectors already grouped
	newClusterIndex = 0;

	for i = 1:nbPts % we tested that result is independent of order
	%   disp(i);
       if usedPts(i) == 0
		usedPts(i) = 1; % flag pt being selected, so it is not counted twice in centroid
		
		[neighborList, usedPts] = findTreeNeighbors2(i, usedPts, neighborIndices);
		indexList = [i; neighborList]; % keep track of which clusters get absorbed
        
        densityList = clusterDensities(indexList);
        highestDensity = max(densityList);
        bestLocalIndices = find(densityList == highestDensity);
        bestIndices = indexList(bestLocalIndices); % back to original indices

        peaks = shiftList(bestIndices,:);
        nbPeaks = length(bestIndices);
        if nbPeaks>1
            disp(['Nb of peaks ', int2str(nbPeaks)]);
        end;
        centroid = sum(peaks,1) / nbPeaks;
%        bestIndexOfLocalList = bestIndices(1);
%        bestIndex = indexList(bestIndexOfLocalList);
%        peak = shiftList(bestIndex,:);
		newClusterIndex = newClusterIndex + 1;
		clusterCenters(newClusterIndex,:) = centroid;
        
		nbDependentClusters = length(indexList); % list contains at least one center, i
		for k = 1:nbDependentClusters % at least 1
		   index = indexList(k);
		   clusterTrace(index) = newClusterIndex; % from many to as many or less
		end; % for
       end % if
	end % for
    clusterCenters = clusterCenters(1:newClusterIndex, :);
	
end; % if
toc
