/*=================================================================
 *
 * findShiftFlags.c  
 *
 * Usage: shiftFlags = findShiftFlags(oldShiftList, newShiftList, quantizError);
 * Inputs:
 * 1. oldShiftList, old centroids
 * 2. newShiftList, new centroids
 * 3. quantizError, error under which old and new centroids are considered the same
 *
 * Outputs:
 * shiftFlags, 1 if the new centroids moved with respect to the old ones
 *
 * This is a MEX-file for MATLAB.  
 * Copyright 2001 Daniel DeMenthon & University of Maryland 
 *
 *=================================================================*/


#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mex.h"

/********************************************************************************/
/*******************************************************************************/

void mexFunction( int nlhs, mxArray *plhs[], 
		  int nrhs, const mxArray *prhs[] )     
{
  int i, j, nbPts, nbFeatures;
  int coordIndex, isCentroidMoved;
  double quantizError;
  double oldCoord, newCoord, delta;
  double *oldShiftList, *newShiftList, *shiftFlags;
 
  
  if(nrhs != 3){
    mexErrMsgTxt("Three inputs required: oldShiftList, newShiftList, quantizError");
  }
  if(nlhs != 1){
    mexErrMsgTxt("One output required: shiftFlags");
  }

  nbPts = mxGetM(prhs[0]); /* nb of centroids */
  nbFeatures = mxGetN(prhs[0]); /* nb of feature components */
  
  oldShiftList = mxGetPr(prhs[0]);
  newShiftList = mxGetPr(prhs[1]);
  quantizError = mxGetScalar(prhs[2]);

  plhs[0] = mxCreateDoubleMatrix(nbPts, 1, mxREAL);
  shiftFlags = mxGetPr(plhs[0]);

  for(i=0;i<nbPts;i++){
    isCentroidMoved = 0;
    for(j=0;j<nbFeatures;j++){ // init coords to zero
      coordIndex = j * nbPts + i;
      oldCoord = *(oldShiftList + coordIndex);
      newCoord = *(newShiftList + coordIndex);
      delta = fabs(newCoord - oldCoord);
      if(delta > quantizError){
	isCentroidMoved = 1;
	break; // a single coordinate change is enough to imply that centroid moved
      }
    }
    shiftFlags[i] = isCentroidMoved;
  }
}


/*******************************************************************************/
