function [clusterList, newTrace] = findTreeShiftedVectors(featureList, weights, treeData, radius, quantizError, doGaussian)
%
% Usage: [clusterLis, newTrace] = findTreeShiftedVectors(featureList, weights, treeData, radius, quantizError)
%
% featureList is the background over which we do the mean shift. IT DOES NOT CHANGE
% There are two types of points: those that shift and those that don't. 
% They are differentiated by shiftFlags
% Points that superpose are grouped and the groupings are mean-shifted
% Points that are grouped are still shifting and are therefore flagged
% Can a shifting point merge with a non-shifting one?
% Can points that didn't move before move again? NO, EVEN IF THEY BECOME DUPLICATE 
% The reason is that the shiftings use only the background, not other shifted pointss
% Merging of duplicates should use all the points including those that stopped moving
% Indeed most mergings occur at cluster centers when shifting is about to stop
% Steps:
% 1. Shift points that moved at the previous iteration
% 2. Merge duplicates among shifted points
% 3. 
% This implementation uses ATRIA tree for finding points within range
% 09/10/01: Use tree data computed here to find nb of pts within ranges of clusters
% (clusterDensities) so that we can later select bestest clusters among neighbors
% dfd 04/24/02: Gaussian weighting of points inside range spheres

nbVects = size(featureList,1);
shiftList = featureList; % List of shifted and merged feature vectors
shiftFlags = ones(nbVects,1); % keep track of which shift vectors to keep processing
movingList = shiftList;
fixedList = [];

trace = [1:nbVects]';
step = 0;
converged = 0;

while ~converged
	step = step + 1;
    disp(' '); % blank line
    disp(['Mean shift step = ', int2str(step)]);
	disp(['Nb of shifting points ', int2str(size(shiftList,1))]);
	disp('Mean shift for moving points');
    if step > 30
        disp(' stop right here');
    end
    [newShiftList, newShiftFlags] = fastShiftTreeFeatures(shiftList, featureList, weights, treeData, radius, quantizError, doGaussian); % one mean shift step for all features
    % newShiftList includes points that did not move, as shown by newShiftFlags
   	movingIndices = find(newShiftFlags);
   	movingList = newShiftList(movingIndices,:);
   	nbMovingPts = length(movingIndices);
   	fixedIndices = find(~newShiftFlags);
   	nbFixedPts = length(fixedIndices);
   	nbPts = length(newShiftFlags);
    prevTotalNbFixedPts = size(fixedList,1);
   	fixedList = [newShiftList(fixedIndices,:);fixedList];
    disp(['Nb of stopped points ', int2str(size(fixedList,1) - prevTotalNbFixedPts)]);
    trace1 = zeros(nbPts,1);
   	trace1(movingIndices) = 1:nbMovingPts;
	trace1(fixedIndices) = nbMovingPts+1:nbPts;

    disp('Merge duplicates');
    % Merge ONLY SHIFTING POINTS
    % Reason is that a shifting point that merges to a fixed point is going to stop and be dropped at nex step anyway
    % Returned weights are not used here but they were useful in previous use of fastMergeTreeDups

    if nbMovingPts>0
	    [mergedMovingList, newWeights, newTrace] = fastMergeDups(movingList, quantizError);
        disp(['Nb of merged points ', int2str(size(movingList,1)-size(mergedMovingList,1))]);
    else % no point left
        mergedMovingList = movingList;
        newTrace = [];
    end; % if
	nbMergedMovingPts = size(mergedMovingList,1);
	nbMergedPts = nbMergedMovingPts + nbFixedPts;
	trace2 = [newTrace; [nbMergedMovingPts+1:nbMergedPts]'];
	mergedTrace = mergeTraces(trace1, trace2);
    mergedTrace = [mergedTrace; [nbMergedPts+1:nbMergedPts+prevTotalNbFixedPts]']; % bottom of trace points to previous fixed pts
%    lengthMergedTrace = length(mergedTrace)
%    maxTrace = max(trace)
    trace = mergeTraces(trace, mergedTrace); % index map between pixels and clusters
    
    shiftList = mergedMovingList;
    	
	converged = (nbMergedMovingPts == 0);
	
end; % while

% Similar points could have been added to the fixed list without merging; merge now

disp('Final merging of shiftList points');
[clusterList, newWeights, tempTrace] = fastMergeDups(fixedList, quantizError);
newTrace = mergeTraces(trace, tempTrace);


