function [oldestClusters, oldestTrace, oldestAges] = keepOldestRegions(newClusters, trace)

% find the clusters and trace for regions with the longest life 
% during the hierarchical mean shift process

persistent bestClusters bestTrace ages bestAges childTrace;
persistent allocatedLabels isBestTraceUpdated savedFirstTrace;

parentTrace = trace;
if isempty(savedFirstTrace)
	bestTrace = trace; % this sets zeros for pixels with no clustering
    bestClusters = newClusters;
	nbBestClusters = max(bestTrace);
    disp(['>>> nb of initial clusters = ',num2str(nbBestClusters),' <<<<']);
	bestAges = ones(nbBestClusters,1); % ages of best clusters
	ages = ones(nbBestClusters,1); % ages of initial clusters
    allocatedLabels = nbBestClusters; % every cluster has distinct label in first trace
    isBestTraceUpdated = zeros(nbBestClusters,1);
    savedFirstTrace = 1;
else % after first pass through previous if
	nbParentClusters = max(parentTrace); % note: this would not work if we relabeled all traces
	for j = 1:nbParentClusters % note: max(parentTrace) changes within this loop
        localCluster = newClusters(j,:);
		parentIndices = find(parentTrace == j); % pixels that have label j in parents
		childRegionLabels = childTrace(parentIndices); % find cluster labels in child for those pixels
        maxLabel = max(childRegionLabels);
        minLabel = min(childRegionLabels);
		if maxLabel == minLabel % then single label in child, no grouping, age of cluster is increased
			clusterLabel = maxLabel;
			parentTrace(parentIndices) = clusterLabel; % replace parent label with index of child, so we can track age
            ages(clusterLabel) = ages(clusterLabel) + 1;
			if isBestTraceUpdated(clusterLabel) % bestTrace has been updated for this cluster
				bestAges(clusterLabel) = bestAges(clusterLabel) + 1;            
            else % bestTrace has not been updated yet for this cluster
                bestLocalLabels = bestTrace(parentIndices);
                bestLocalAges = bestAges(bestLocalLabels);
                oldestAge = max(bestLocalAges); % find oldest cluster overlapped by maxLabel cluster
%  	        if ages(clusterLabel) >= oldestAge
	        if ages(clusterLabel) > oldestAge
% 	        if ages(clusterLabel) > sqrt(oldestAge)
%             if ages(clusterLabel) > log(oldestAge) - 1
				bestClusters(clusterLabel,:) = localCluster;
				bestTrace(parentIndices) = clusterLabel; % place clusterLabel in pixels of bestTrace
				bestAges(clusterLabel) = ages(clusterLabel);
				isBestTraceUpdated(clusterLabel) = 1;
            	end;
            end;
        else  % a grouping occurred
            allocatedLabels = allocatedLabels + 1; % a new label is allocated; recycle old one instead?
            parentTrace(parentIndices) = allocatedLabels; % a new label for the grouping
            ages(allocatedLabels) = 1; % no  point checking ages of bestTraces
            isBestTraceUpdated(allocatedLabels) = 0; 
        end; % if maxLabel == minLabel
	end; % for j
end % else isSavedTrace
childTrace = parentTrace;

oldestClusters = bestClusters;
oldestTrace = bestTrace;
oldestAges = bestAges;

