function [normalizedFeatures, ranges] = normalizeFeatures(featureList, importances)
%
% Usage: [normalizedFeatures, ranges] = normalizeFeatures(featureList, importances)
% map feature coordinates to [0-1] range
% but reduce range according to importances, which are numbers between 0 and 1
%
[nbPts, nbFeatures] = size(featureList);
normalizedFeatures = featureList; % copy zero terms

lo = 1; hi = 2;

for i = 1:nbFeatures
    compMin = min(featureList(:,i));
    compMax = max(featureList(:,i));
    delta = compMax-compMin;
	if delta ~= 0 % else copy above is enough
	    normalizedFeatures(:,i) = importances(i) * (featureList(:,i) - compMin)/delta;
	else % delta = 0, set within hypercube 1
	    normalizedFeatures(:,i) = featureList(:,i)/compMax;
	end;
	ranges(i,lo) = compMin;
	ranges(i,hi) = compMax;
end;
