% function [newClusterCenters, newClusterTrace] = spaceTimeMeanShift(rootName, initFrame, endFrame, showPictures)
%
% Usage: [newClusterCenters, newClusterTrace] = spaceTimeMeanShift(rootName, initFrame, endFrame, showPictures)
%
%
%if Output is cluster centers found from mean shift analysis 

% Tasks for space-time segmentation of video stack
%
% 1. Read all frames and put in video stack
%
% 2. Read optical flow images and masks showing where optical flow is defined
%
% 3.  Find stack of projection distance images
%
% 4. Make feature vector list
%
% 5. Do hierarchical mean shift and scale-space in pts of feature vector space
%

% =================================================================================
% =================================================================================

if 0
 rootName = '/dfs/homes/daniel/MacTreeMeanShift3D_2/HandPaper/handPaper_';
 initFrame = 16; endFrame = 35;
 scale = 0.25;
end

if 0
 rootName = '/dfs/homes/daniel/MacTreeMeanShift3D_2/TableTennis/tableTennis_';
 initFrame = 1; endFrame = 23;
 scale = 0.25;
end

if 0
 rootName = '/dfs/homes/daniel/MacTreeMeanShift3D/VACE/DrugDeal-DVCam_';
 initFrame = 1366; endFrame = 1385;
%  initFrame = 1365; endFrame = 1388;
end

if 0
 rootName = '/dfs/homes/daniel/MacTreeMeanShift3D/Soccer/soccer_1_';
 % initFrame = 28; endFrame = 46;
 initFrame = 7; endFrame = 26;
%  initFrame = 7; endFrame = 30;
 scale = 0.25;
%  scale = 0.5; % not good at all: FIX
end

if 0
 rootName = '/dfs/homes/daniel/Matlab/SpaceTime3D/SmallFlowerGardenFrames/sflowg';
 nbFrames = 20; % nbFrames - 8 remain after optical flow calculation
% nbFrames = 24; % nbFrames - 12 remain after smoother optical flow calculation
% nbFrames = 40; % nbFrames - 8 remain after optical flow calculation with less smoothing

end

if 0 % setup to read frames from CD ROM sequence
   	rootName = '/fs/lamp/Projects/VACE/VACEData/LAMPSequences/Seq13/Segment/Seq13_';
	initFrame = 360; endFrame = 378;
	top = 110; right = 295; bottom = 180; left = 213;
end;

if 1 % other paths are remnants from older tests
 rootName = '/fs/lamp/Projects/SpaceTimeSeg/sflowgFolder/sflowg';
 initFrame = 7; 
 endFrame = 22;
end

tic

% =================================================================================

% 1. Read all frames and put in video stack
disp('Read stack of images: color and motion');
[rgb3D, angleMapX, angleMapY, vDefined] = readAllImages(rootName, initFrame, endFrame);

[nbFrames, nbRows, nbCols] = size(angleMapX); % these are trimmed dimensions
nbPixels = nbFrames * nbRows * nbCols;

nbUndefinedPixels = nbPixels - sum(vDefined(:));
if nbUndefinedPixels > 0.9 * nbPixels 
    newClusterCenters = [];
    newClusterTrace = [];
    disp(' %%%% Optical flow is too sparse; sequence is probably black %%%%');
    return;
end

% 2. Find projection maps
disp('Find projection maps');


projDistOffsetX  = ceil(sqrt(nbFrames^2 + nbCols^2)/2) + 1;
projDistOffsetY  = ceil(sqrt(nbFrames^2 + nbRows^2)/2) + 1;

[projMapX, projMapY] = makeProjMaps(angleMapX, angleMapY, projDistOffsetX, projDistOffsetY, vDefined);

[frameCountBlock, rowCountBlock, colCountBlock] = makeCountBlocks(nbFrames, nbRows, nbCols);

% 3. Make LUV images
disp('Make LUV image stack');
% [rImage, gImage] = getRGImages3D(rgb3D);
[LImage,uImage,vImage] = getLuvImages3D(rgb3D);

% 4. Make feature vector list
disp('Make list of feature vectors');

% featureList = [rowCountBlock(:), colCountBlock(:), LImage(:), uImage(:), vImage(:)]; % 5 components
% featureList = [angleMapX(:), angleMapY(:), rowCountBlock(:), colCountBlock(:), LImage(:), uImage(:), vImage(:),frameCountBlock(:)]; % 8 components
featureList = [angleMapX(:), angleMapY(:), projMapX(:), projMapY(:), LImage(:), uImage(:), vImage(:),frameCountBlock(:)]; % 8 components
% featureList = [angleMapX(:), angleMapY(:), projMapX(:), projMapY(:), LImage(:), uImage(:), vImage(:)]; % 7 components
% featureList = [angleMapX(:), angleMapY(:), LImage(:), uImage(:), vImage(:)]; % 5 components
% featureList = [angleMapX(:), angleMapY(:), projMapX(:), projMapY(:)]; % 4 components
% featureList = [LImage(:), uImage(:), vImage(:)]; % 3 components
% featureList = [angleMapX(:), angleMapY(:)]; % 2 components
% featureList = [angleMapX(:), angleMapY(:), projMapX(:), projMapY(:), grey3D(:)]; % 5 components

nbPts = length(featureList) % for display only

dataFlags = vDefined(:);


% 8. Run MeanShiftEngine

disp('Start Mean Shift Engine'); 

time1 = cputime;

% radius = 1/4; % quantizError defined in meanShiftMultiPass
% 	radius = 1/5; % quantizError defined in meanShiftMultiPass
% 	radius = 1/8; % quantizError defined in meanShiftMultiPass
radius = 1/6; % quantizError defined in meanShiftMultiPass
% importances = [0.8,0.8,0.6,0.6,1,1,1];
importances = [1,1,1,1,1,1,1,0.2];
% importances = [1,1,1,1,0.5,0.5,0.5];
% radius = 1/7; % quantizError defined in meanShiftMultiPass
% importances = [0.5,0.5,0.25,0.25,1,1,1];
doGaussian = 1;
% doGaussian = 0;
% [clusterCenters, clusterTrace, closeVectIndexList, ranges] = meanShiftMultiPass(featureList, dataFlags, radius, importances, doGaussian);
[clusterCenters, clusterTrace, closeVectIndexList, ranges] = meanShiftOptimalTrace(featureList, dataFlags, importances, doGaussian); 

time2 = cputime;
clusteringTime = time2 - time1;
disp(['****** clustering time ',num2str(clusteringTime),' ******']);
disp(['---- Nb of regions ', num2str(max(clusterTrace)), ' ----']);

% Testing computation of optical flow from color strands
if 0 
newClusters = updateAnglesAndDists(clusterCenters, clusterTrace, [nbFrames,nbRows,nbCols]);
clusterTrace3D = reshape(clusterTrace, [nbFrames,nbRows,nbCols]);
[angleX, angleY] = segmentMotionsFromClusters(newClusters(:,1), newClusters(:,2), clusterTrace3D, 0);
samplePixelBlock(angleX,1);
end
% Find cluster colors
% clusterColors = findClusterColors3D(rgb3D, clusterTrace);

clusterColors = findClusterColors3D2(rgb3D, closeVectIndexList);
if 0
    clusterColors = findAverageClusterColors(featureList, clusterTrace); % average luv colors
    clusterColors = findRGBClusterColorsFromLUV(clusterCenters); % average luv colors
end;

% Find segment labels
% Make clusterTrace a 3D stack giving cluster label for each pixel of stack
disp('Label pixels by reshaping cluster trace to video stack shape');
clusterTrace3D = reshape(clusterTrace, nbFrames, nbRows, nbCols);

% coloredClusterTrace = colorEachLabel2(clusterTrace3D);
coloredClusterTrace = colorEachLabel3(clusterTrace3D);
% coloredClusterTrace = colorEachLabel(clusterTrace3D);
samplePixelBlock(coloredClusterTrace, 1);

% Find line segments corresponding to clusters
% Use cluster trace to find only portions of cluster lines for which there are contributing pixels in the same row
disp('Find line segments along cluster lines');
clusterSegments =  findClusterSegments3D(clusterCenters, clusterTrace3D);

% Segment shot stack by giving pixels the values of the clusters which they clustered to
disp('Find color and angle segmentations');
[segmented3D, angle3DX, angle3DY] = segmentShotsFromClusters3D(clusterCenters, clusterColors, clusterTrace3D);
segmentedBorders = colorsFromClusters3D(clusterColors, clusterTrace3D);
samplePixelBlock(segmented3D,1);
samplePixelBlock(segmentedBorders,1);
samplePixelBlock(angle3DX,1);
% samplePixelBlock(angle3DY,1);

% Keep only clusters for which average nb of pixels per row is large
disp('Sort clusters by stripe thickness');

% minNbPix = 10; % segments with less than minNbPix are not used as features
minNbPix = 20; % segments with less than minNbPix are not used as features

% bestIndexList = pickBestClusters3D(clusterCenters, clusterSegments, minNbPix);
bestIndexList = pickBestClusters3D(clusterSegments, minNbPix);

bestClusterSegments = clusterSegments(bestIndexList,:);
bestClusterCenters = clusterCenters(bestIndexList,:);
bestClusterColors = clusterColors(bestIndexList,:);

clusterTraceBest = zeros(size(clusterCenters,1),1); % pixels pointing to small clusters will get zero trace
clusterTraceBest(bestIndexList) = [1:length(bestIndexList)];
bestClusterTrace = mergeTraces(clusterTrace, clusterTraceBest);
% bestClusterTrace3D = reshape(bestClusterTrace, nbFrames, nbRows, nbCols);
% smallRegionFlags3D = vDefined & (bestClusterTrace3D == 0);
% smallRegionFlags = smallRegionFlags3D(:);


if 0
	smallRegionFlags = vDefined(:) & (bestClusterTrace == 0);
	smallRegionLabels = fuseSmallRegions(smallRegionFeatureList, bestClusterCenters, ranges);
	smallRegionTrace = mergeTraces(smallRegionTrace, smallRegionLabels);
	smallRegionTrace3D = reshape(smallRegionTrace, nbFrames, nbRows, nbCols);
	
	fusedClusterTrace = bestClusterTrace + smallRegionTrace; % when one labeling is zero the other is not and vice versa
	fusedClusterTrace3D = reshape(fusedClusterTrace, nbFrames, nbRows, nbCols);
	fusedColoredClusterTrace = colorEachLabel3(fusedClusterTrace3D);
    samplePixelBlock(fusedColoredClusterTrace,1);

	% [bestSegmented3D, bestAngle3DX, bestAngle3DY] = segmentShotsFromClusters3D(bestClusterCenters, bestClusterColors, bestClusterTrace3D);
	[fusedSegmented3D, fusedAngle3DX, fusedAngle3DY] = segmentShotsFromClusters3D(bestClusterCenters, bestClusterColors, smallRegionTrace3D);
	samplePixelBlock(fusedSegmented3D,1);% show only small regions being fused
	[fusedSegmented3D, fusedAngle3DX, fusedAngle3DY] = segmentShotsFromClusters3D(bestClusterCenters, bestClusterColors, fusedClusterTrace3D);
	samplePixelBlock(fusedSegmented3D,1);
	samplePixelBlock(fusedAngle3DX,1);
	fusedSegmentedBorders = colorsFromClusters3D(bestClusterColors, fusedClusterTrace3D);
	samplePixelBlock(fusedSegmentedBorders,1);
	
	% Add average nbs of pixels per frame that contributed to them
	% and remove offset from distX and distY
	disp('Use stripe thickness instead of cluster mass as feature');
end

featureVectors = makeFeatureVectors3D(bestClusterCenters, bestClusterSegments, projDistOffsetX, projDistOffsetY);

% disp('Save workspace as a *.mat file');
% save spaceTime3DResults;
	
toc
% Now use plot routines to show the lines corresponding to moving objects
% Draw these lines with the right colors in perspective projections
% using intersection between lines and cube boundaries
[lineList, colorList, thicknessList] = listClusterLines3D(nbFrames, nbRows, nbCols, bestClusterCenters, bestClusterSegments, bestClusterColors);
draw3DLines(nbFrames, nbRows, nbCols, lineList, colorList, thicknessList);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if 0
% Segment optical flow field only. clusterTrace3D is used to define borders between regions

	if 1
		motionFeatureList = [angle3DX(:), angle3DY(:)];
	else
		motionFeatureList = [fusedAngle3DX(:), fusedAngle3DY(:)];
	end;
	% dim = 2;
	% radius = getRadius(dim);
	% radius = 1/10;
% 	quantizError = radius/20;
	motionRadius = 1/20;
	disp(['Radius for 2D ', num2str(motionRadius)]);
    motionImportances = [1,1];
    motonDoGaussian = 1;
	% [motionClusters, motionClusterTrace, closeVectIndexList] = meanShiftEngine3(motionFeatureList, dataFlags, radius, quantizError);
    % [motionClusters, motionClusterTrace] = meanShiftEngine3(motionFeatureList, dataFlags, radius, quantizError);
	[motionClusters, motionClusterTrace] = meanShiftMultiPass(motionFeatureList, dataFlags, motionRadius, motionImportances, motionDoGaussian);
	motionClusterTrace3D = reshape(motionClusterTrace, nbFrames, nbRows, nbCols);
	% coloredMotionBlock = colorEachLabel(motionClusterTrace3D);
	% samplePixelBlock(coloredMotionBlock,1);
	
	drawBorders = 1;
	[segmentedAngleX, segmentedAngleY] = segmentMotionsFromClusters(motionClusters(:,1), motionClusters(:,2), motionClusterTrace3D, drawBorders);
	
	samplePixelBlock(segmentedAngleX,1);
	motionSegments = findClusterSegments3D(motionClusters, motionClusterTrace3D);
	
	minNbPix = 50; % segments with less than minNbPix are not used as features
	bestMotionIndexList = pickBestClusters3D(motionSegments, minNbPix);
	
	bestMotionSegments = motionSegments(bestMotionIndexList,:);
	bestMotionClusters = motionClusters(bestMotionIndexList,:);
	
	motionClusterTraceBest = zeros(size(motionClusters,1),1); % pixels pointing to small clusters will get zero trace
	motionClusterTraceBest(bestMotionIndexList) = [1:length(bestMotionIndexList)];
	bestMotionClusterTrace = mergeTraces(motionClusterTrace, motionClusterTraceBest);
	bestMotionClusterTrace3D = reshape(bestMotionClusterTrace, nbFrames, nbRows, nbCols);
	
	for i = 1:4 % show 4 largest motion regions
        singleSegment = segmentSingleCluster(rgb3D, bestMotionClusterTrace3D, i);
        samplePixelBlock(singleSegment,1);
	end;

end % if 0

