function [sortedCounts, sortedActions] =  actionVideoQueryCore(clipTubeDataSet, treeData, actionIndexList, queryClipTubes, k)

% Given a list of tubes from a video database, and the list of corresponding action indices
% Pass a query list of tubes and return the indices of the actions closest to this list of tubes
% 1. Find the indices of the tubes within range of each clipTube
% 2. Find the corresponding indices of the actions those tubes are coming from
% 3. Count votes for each action and return (1) number of votes and (2) action indices in decreasing order of vote

% dataSet = clipTubeDataSet(:,1:4); % remove size; does not work otherwise; remove color
% 
% clipTubes = queryClipTubes(:,1:4); % remove size & color

dataSet = clipTubeDataSet;

clipTubes = queryClipTubes; 

% nearestNeighbors = 0;
nearestNeighbors = 1; % we don't use range search
if nearestNeighbors
    nbStrandReturns = k; % k = 3 in k-NN
%     nbStrandReturns = 1; % k = 1 in k-NN
	[neighborIndices, distances] = nn_search(dataSet, treeData, clipTubes, nbStrandReturns);
% try to add an epsilon relative NN error as last argument to see if we get speedup.
% what is format of neighborIndices? nbRows = tubes in clipTubes, nbCols = nbStrandReturns;
% neighborIndices gives indices of strands close to each of the strands of query video
	flatNeighborIndices = neighborIndices(:); % do not care about ordered choice
else % range search
	maxDist = 10;
	[counts, neighbors] = range_search(dataSet, treeData, clipTubes, maxDist); % one cell of neighbors per clipTube
	neighborIndices = neighbors(:,1); % pass only the part of neighbors that is useful (distances are not)
	flatNeighborIndices = [];
	for i = 1:size(clipTubes,1)
        flatNeighborIndices = [flatNeighborIndices, neighborIndices{i}];
	end;
end % range search

selectedActions = actionIndexList(flatNeighborIndices);

[sortedByIndexActions, indexMap] = sort(selectedActions);
nbIndices = length(sortedByIndexActions);

actionIndex = -1;
index = 0;

% Count votes for each action and return number of votes and action indices in decreasing order of vote

for i = 1:nbIndices
	newActionIndex = sortedByIndexActions(i);
	if newActionIndex == actionIndex
		newIndex = index;
		howManyActions(newIndex) = howManyActions(newIndex) + 1;
	else % we go here if i == 1
		newIndex = index + 1;
		howManyActions(newIndex) = 1;
		whichActions(newIndex) = newActionIndex;
	end; % if
	actionIndex = newActionIndex;
    index = newIndex;
end;    

[sortedCounts, sortedIndices] = sort(-howManyActions); % sort in decreasing order
sortedCounts = - sortedCounts;

sortedActions = whichActions(sortedIndices);


