function [sortedActions, sortedActionNbs, sortedCounts] =  actionVideoQueryFunc(dataStruct, queryStruct, labelChoices, roiList, k)

% Given a video clip name, return closest videos in database
%
% 0. Unpack dataStruct, database composed of list of tubes (one per line) and list of video indices where tubes come from
% 1. Prune and normalize training set
% 2. Make tree, without using the thickness component of the tubes
% 3. Unpack queryStruct, list of tubes for query video
% 4. Find the indices of the tubes within range of each query tube
% 5. Find the corresponding actions of the videos those tubes are coming from
% 6. Count votes for each video and return number of votes and action in decreasing order of vote

% action2RoiListName = ['/fs/lamp/Projects/SpaceTimeSeg/ActionRecognition/ActionMPEGs/action2RoiList.txt'];
% 
% databaseName = ['/fs/lamp/Projects/SpaceTimeSeg/ActionRecognition/ActionResults/mergedDatabase.mat'];
% queryDatabaseName = ['/fs/lamp/Projects/SpaceTimeSeg/ActionRecognition/ActionResults/mergedDatabase.mat'];
% queryDatabaseName = ['/fs/lamp/Projects/SpaceTimeSeg/ActionRecognition/ActionResults/databasepickPhoneD.mat'];
% startFrame = 11; endFrame = 30;
% nbReturns = 5; % k of k-NN

% dataStruct = load(databaseName);

%====================================================================================================================

disp('1. Unpack, prune and normalize training set ');

trainingTubeSet = dataStruct.dataSet;
videoIndexList = dataStruct.videoIndices;
% videoIndexList = videoIndexList(:,1); 
actionLabelList = dataStruct.actionLabels;
actionLabelList = actionLabelList(:,2);

% [labelChoices, roiList] = readAction2RoiList(action2RoiListName); % read list of actions and corresponding roi rectangles
prunedIndexList = filterTrainingTubes(trainingTubeSet, actionLabelList, labelChoices, roiList); % indices of tubes who fit in their rois

trainingTubeSet = trainingTubeSet(prunedIndexList,1:4);
videoIndexList = videoIndexList(prunedIndexList,:);
trainingLabelList = actionLabelList(prunedIndexList);
trainingActionNbList = label2Nb(labelChoices, trainingLabelList); % use indices instead of names such as push, put. 

% importances = [1, 1, 1, 1];
importances = [1, 1, 0.3, 0.3];
[normalizedTrainingTubeSet, ranges] = normalizeFeatures(trainingTubeSet, importances);

disp('2. Make tree ');
treeData = nn_prepare(normalizedTrainingTubeSet); 


disp('3. Unpack, prune and normalize query set ');

% queryStruct = load(queryDatabaseName);

queryTubeSet = queryStruct.dataSet;
queryVideoIndexList = queryStruct.videoIndices;
% queryVideoIndexList = queryVideoIndexList(:,1); % we may actually need both video index and frame number
queryLabelList = queryStruct.actionLabels;
queryLabelList = queryLabelList(:,2);

prunedQueryIndexList = filterQueryTubes(queryTubeSet, roiList); % indices of tubes who fit in any of the rois

queryTubeSet = queryTubeSet(prunedQueryIndexList,1:4); % no colors are used
queryVideoIndexList = queryVideoIndexList(prunedQueryIndexList,:);
queryLabelList = queryLabelList(prunedQueryIndexList);
queryActionNbList = label2Nb(labelChoices, queryLabelList); % use indices instead of names such as push, put. 

nbQuerySetTubes = size(queryTubeSet,1);
normalizedQueryTubeSet = normalizeToRanges(queryTubeSet, ranges, importances); % normalize to same range as traning features

disp('4. Pass query set of tubes to actionVideoQueryCore ');

videoNb = queryVideoIndexList(1,1); % this assumes all tubes of query are from same video
frameNb = queryVideoIndexList(1,2); % this assumes all tubes of query are from same video
disp(['Video # ', num2str(videoNb), ', frame # ',num2str(frameNb), ', Action ', char(queryLabelList(1))]);
disp(['Number of query tubes ', num2str(nbQuerySetTubes)]);
[sortedCounts, sortedActionNbs] =  actionVideoQueryCore(normalizedTrainingTubeSet, treeData, trainingActionNbList, normalizedQueryTubeSet, k);
sortedActions = labelChoices(sortedActionNbs);
return;

