% function [sortedVideos, sortedFrames] =  actionVideoQueryTester(databaseName, queryDatabaseName, nbReturns)

% Given a video clip name, return closest videos in database
%
% 1. Read database composed of list of tubes (one per line) and list of video indices where tubes come from
% 2. Make tree again, as we don't use the thickness component of the tubes
% 3. Take a name of a video clip, start and end frames and get clipTubes array
% 4. Find the indices of the tubes within range of each clipTube
% 5. Find the corresponding indices of the videos those tubes are coming from
% 6. Count votes for each video and return number of votes and video name in decreasing order of vote

action2RoiListName = ['/fs/lamp/Projects/SpaceTimeSeg/ActionRecognition/ActionMPEGs/action2RoiList.txt'];

databaseName = ['/fs/lamp/Projects/SpaceTimeSeg/ActionRecognition/ActionResults/mergedDatabase.mat'];
queryDatabaseName = ['/fs/lamp/Projects/SpaceTimeSeg/ActionRecognition/ActionResults/mergedDatabase.mat'];
% queryDatabaseName = ['/fs/lamp/Projects/SpaceTimeSeg/ActionRecognition/ActionResults/databasepickPhoneD.mat'];
% startFrame = 11; endFrame = 30;
% nbReturns = 5; % k of k-NN

dataStruct = load(databaseName);

trainingTubeSet = dataStruct.dataSet;
% treeData = dataStruct.treeData;
videoIndexList = dataStruct.videoIndices;
% videoIndexList = videoIndexList(:,1); 
actionLabelList = dataStruct.actionLabels;
actionLabelList = actionLabelList(:,2);

[labelChoices, roiList] = readAction2RoiList(action2RoiListName); % read list of actions and corresponding roi rectangles
prunedIndexList = filterTrainingTubes(trainingTubeSet, actionLabelList, labelChoices, roiList); % indices of tubes who fit in their rois

trainingTubeSet = trainingTubeSet(prunedIndexList,1:4);
videoIndexList = videoIndexList(prunedIndexList,:);
trainingLabelList = actionLabelList(prunedIndexList);
trainingActionNbList = label2Nb(labelChoices, trainingLabelList); % use indices instead of names such as push, put. 

importances = [1, 1, 1, 1];
[normalizedTrainingTubeSet, ranges] = normalizeFeatures(trainingTubeSet, importances)
treeData = nn_prepare(normalizedTrainingTubeSet); 


queryStruct = load(queryDatabaseName);

queryTubeSet = queryStruct.dataSet;
queryVideoIndexList = queryStruct.videoIndices;
% queryVideoIndexList = queryVideoIndexList(:,1); % we may actually need both video index and frame number
queryLabelList = queryStruct.actionLabels;
queryLabelList = queryLabelList(:,2);

prunedQueryIndexList = filterQueryTubes(queryTubeSet, roiList); % indices of tubes who fit in any of the rois

queryTubeSet = queryTubeSet(prunedQueryIndexList,1:4);
queryVideoIndexList = queryVideoIndexList(prunedQueryIndexList,:);
queryLabelList = queryLabelList(prunedQueryIndexList);
queryActionNbList = label2Nb(labelChoices, queryLabelList); % use indices instead of names such as push, put. 

nbQuerySetTubes = size(queryTubeSet,1);
normalizedQueryTubeSet = normalizeToRanges(queryTubeSet, ranges, importances); % normalize to same range as traning features

correctAnswerCount = 0;
incorrectAnswerCount = 0;
tubeCount = 0;

for i = 1:nbQuerySetTubes
    videoNb = queryVideoIndexList(i,1);
    frameNb = queryVideoIndexList(i,2);
    if (i==1) | (videoNb ~= prevVideoNb) | (frameNb ~= prevFrameNb) % new video or new subset of video
       queryClipTubes = [normalizedQueryTubeSet(i,:)];
       startIndex = i;
    else % continuing video
        queryClipTubes = [queryClipTubes; normalizedQueryTubeSet(i,:)]; % OK if i == 1 
    end;
    isEndOfVideoPiece = (i == nbQuerySetTubes);
    if i ~= nbQuerySetTubes
        nextVideoNb = queryVideoIndexList(i+1,1);
        nextFrameNb = queryVideoIndexList(i+1,2);
        isEndOfVideoPiece = videoNb ~= nextVideoNb | frameNb ~= nextFrameNb; % neve reaches here if nbQuerySetTubes == 1
    end;

    if isEndOfVideoPiece % we just exited to another clip or finished reading, we have a query to perform with the previous set of tubes
        disp(['Index # ', num2str(startIndex), ', Video # ', num2str(videoNb), ', frame # ',num2str(frameNb), ', Action ', char(queryLabelList(i))]);
        disp(['Number of query tubes ', num2str(length(queryClipTubes))]); 
        [sortedCounts, sortedActionNbs] =  actionVideoQueryCore(normalizedTrainingTubeSet, treeData, trainingActionNbList, queryClipTubes);
%         [sortedCounts, sortedVideos, sortedFrames, sortedIndices] =  videoQueryCore(trainingTubeSet, treeData, videoIndexList, queryClipTubes);
%         sortedActions = trainingLabelList(sortedIndices);
%         disp([char(sortedActions), repmat('  ', size(sortedActions)), num2str(sortedCounts')]);
        sortedActions = labelChoices(sortedActionNbs);
        disp([char(sortedActions), repmat('  ', size(sortedActionNbs')), num2str(sortedCounts')]);
        bestAction = sortedActions(1);
%       bestNamedAction = labelChoices(bestAction);
        actualAction = queryLabelList(i);
        if strcmp(bestAction, actualAction)
            correctAnswerCount = correctAnswerCount + 1;
        else
            incorrectAnswerCount = incorrectAnswerCount + 1;
        end
    end
    
    prevVideoNb = videoNb;
    prevFrameNb = frameNb;
end

correctAnswerCount
incorrectAnswerCount
proportionCorrect = 100 * correctAnswerCount / (correctAnswerCount + incorrectAnswerCount);
disp(['Proportion of correct answers is ', num2str(proportionCorrect), ' %']);

