% function leaveOneOut

% Read fileList, make database of tubes for all videos except one;
% Use that one has a query video and compare its action label with the label found by nearest neighbors
%
% Output confusion matrix

% 1. Define paths

actionDir = ['./ActionData'];
mpegDir = ['ActionMPEGs'];
resultsDir = ['ActionResults'];
mergedDatabaseName = 'mergedDatabase';

trainingDatabaseName = [actionDir,'/', resultsDir, '/', mergedDatabaseName];
fullFileListName = [actionDir, '/', mpegDir, '/fileList.txt'];
action2RoiListName = ['./ActionData/ActionMPEGs/action2RoiList.txt'];

% 2. Read fileList.txt and list of action & rects

[nameList, startFrameList, endFrameList, frameCountList, labelList] = readVideoList(fullFileListName);
[labelChoices, roiList] = readAction2RoiList(action2RoiListName); % read list of actions and corresponding roi rectangles

labelNbList = label2Nb(labelChoices, labelList); % used for confusion matrix
nbChoices = length(labelChoices);
confusionMatrix = zeros(nbChoices, nbChoices);
voteMatrix = zeros(nbChoices, nbChoices); % a matrix in which the votes for different actions are cumulated

k = 3; % k neighbors in kNN

% 3. Loop over each available mpeg file, read corresponding database and merge all other databases, excluding database of mpeg
correctAnswerCount = 0;
incorrectAnswerCount = 0;

% for i = 1:length(nameList)-2 % don't include bottle actions in testing 
for i = 1:length(nameList)
% for i = 30:31 % testing pickBottle and dropBottle
	clipName = char(nameList(i));
	queryDatabaseName = [actionDir,'/', resultsDir, '/', 'database', strtok(clipName, '.')];
    queryStruct = load(queryDatabaseName);
	
	skippedIndex = i;
	mergeDatabasesFunc(actionDir, mpegDir, resultsDir, skippedIndex, trainingDatabaseName); % create merged database
    trainingDataStruct = load(trainingDatabaseName);
	[sortedActions, sortedActionNbs, sortedCounts] =  actionVideoQueryFunc(trainingDataStruct, queryStruct, labelChoices, roiList, k); % classification by k-NN
    disp([char(sortedActions), repmat('  ', size(sortedCounts')), num2str(sortedCounts')]);
	bestAction = sortedActions(1);
    bestActionNb = sortedActionNbs(1);
	actualAction = labelList(i);
    actualActionNb = labelNbList(i);
	if bestActionNb == actualActionNb % faster than comparing strings
	    correctAnswerCount = correctAnswerCount + 1;
	else
	    incorrectAnswerCount = incorrectAnswerCount + 1;
	end
    
    confusionMatrix(actualActionNb, bestActionNb) = confusionMatrix(actualActionNb, bestActionNb) + 1;
    row = zeros(1, nbChoices);
    row(sortedActionNbs) = sortedCounts;
    voteMatrix(actualActionNb,:) = voteMatrix(actualActionNb,:) + row; % accumulate all the action votes 
end;

sumConfusionRows = sum(confusionMatrix, 2);
normalizer = repmat(sumConfusionRows, [1,nbChoices]);
normalizedConfusionMatrix = confusionMatrix ./ normalizer;

correctAnswerCount
incorrectAnswerCount
proportionCorrect = 100 * correctAnswerCount / (correctAnswerCount + incorrectAnswerCount);
disp(['Proportion of correct answers is ', num2str(proportionCorrect), ' %']);
normalizedConfusionMatrix

voteMatrix
