% function [dataSet, videoIndices] = setStrandDatabase
%
%
%
% Path to files: ActionData
% List of mpeg sequences is in fileList.txt
% fileList.txt contains mpg name, start frame, end frame, nb of frames, and an action label
% There is a directory made for each sequence, using prefix in front of .mpg
% In this directory, we make 3 directories
% QuarterSize containing bicubic reductions of frames in bmp format
% MotionX and MotionY containing angleX and angleY of motion vectors
%
% 1. Read mpeg file name and frame count in fileList.txt located in ActionRecognition
% 2. Create directory with prefix of mpeg file, and subdirectories QuarterSize, MotionX and MotionY
% 3. Fill QuarterSize with 1/4 reduced RGB frames, 
%    with 6 more frames at beginning and end for fitering purposes
% 4. Fill MotionX and MotionY with optical flow results
% 5. Read frames of 3 subdirectories in small chunks 
%    including extra frames at both ends to account for files lost in filtering
% 6. Find tubes for each chunk using these files 
% 7. Store the 8 components of each tube as row of matrix (including thickness, which is too unreliable and is not used)
% 8. Also store the index of the video where the tube comes from in a separate list called videoIndices

% scale = 0.25; % scale reduction of original RGB images
scale = 0.125; % scale reduction to 90 x 60 for mpegs made from higher quality 720 x 480 Quicktime movies
frameSpan = 6; % extra frames needed because of filtering
processedClipSize = 12;
% processedClipSize = 7; % smaller clips for testing database
% processedClipSize = 16;
actionDir = ['ActionData'];
mpegDir = ['ActionMPEGs'];
resultsDir = ['ActionResults'];

% actionList = {'pan'; 'pull'; 'put'; 'push'; 'take'};

fullFileListName = [actionDir, '/', mpegDir, '/fileList.txt'];
[nameList, startFrameList, endFrameList, frameCountList, labelList] = readVideoList(fullFileListName);


prevClipName = '';
% beginIndex = 1;
% endIndex = 29;
beginIndex = 1;
endIndex = 1; % for testing

for i = beginIndex:endIndex 

    clipName = char(nameList(i));
    labelName = char(labelList(i));
    
    if i ~= endIndex
        nextClipName = char(nameList(i+1));
    end
    if i == beginIndex | strcmp(clipName, prevClipName) == 0
		dataSet = [];
		videoIndices = [];
		actionLabels = {};
    end
    
    namePiece = [strtok(clipName, '.'), '_']; % prefix of mpeg name + _
    subDirName = [namePiece, labelName]; % name of subdirectory of actions where motion & rgb info are kept
    clipDir = [actionDir, '/', subDirName]; % used as parent of following subdirectories
    rgbDir = ['RGB'];
    rgbDir2 = [subDirName, '/', 'RGB']; % needed to go from ActionDir to RGB when reading MPEG clips
    scaledDir = ['QuarterSize'];
    motionXDir = ['MotionX'];
    motionYDir = ['MotionY'];
    vDefinedDir = ['VDefined'];

    startFrame = startFrameList(i);
    endFrame = endFrameList(i);
    frameCount = endFrame - startFrame + 1;
    if ~exist(clipDir, 'dir')
        disp('Make directories');
      	[status, msg] = mkdir(actionDir, subDirName); % make directory with prefix as name
      	[status, msg] = mkdir(clipDir, rgbDir); % make directory with prefix as name
      	[status, msg] = mkdir(clipDir, scaledDir); % make directory with prefix as name
      	[status, msg] = mkdir(clipDir, motionXDir); % make directory with prefix as name
      	[status, msg] = mkdir(clipDir, motionYDir); % make directory with prefix as name
      	[status, msg] = mkdir(clipDir, vDefinedDir); % make directory with prefix as name
        disp('Make PPMs from MPEGs');
	    makePPMsFromMPEG(actionDir, mpegDir, rgbDir2, clipName, startFrame-frameSpan, endFrame+frameSpan); % PPMs in rgbDir
        disp('Make scaled BMPs');
	    convertAndScaleImages(clipDir, rgbDir, scaledDir, namePiece, startFrame-frameSpan, endFrame+frameSpan, scale); % scaled PPMs in scaledDir
        disp('Compute and store optical flows');
        makeAffineFlowImages(clipDir, scaledDir, motionXDir, motionYDir, vDefinedDir, namePiece, startFrame, endFrame);
    end
%   makeAffineFlowImages(clipDir, scaledDir, motionXDir, motionYDir, vDefinedDir, namePiece, startFrame, endFrame);

% if 0 % complete later  
% 2. Find tubes and store them in database
%     nbPieces = floor(frameCount/processedClipSize);
    initFrame = startFrame; % initial and final frames for pieces
    while 1
        finalFrame = initFrame + processedClipSize - 1;
        if finalFrame > endFrame
            finalFrame = endFrame;
            if finalFrame - initFrame + 1 < 8 % not enough frames left
                break;
            end
        end
        tic
	    disp(['startFrame = ', num2str(initFrame), ', endFrame = ', num2str(finalFrame)]);
        clipTubes = findAffineFlowClipTubes(clipDir, scaledDir, motionXDir, motionYDir, vDefinedDir, ...
						namePiece, initFrame, finalFrame, 1); % showPictures = 1
        toc
        if ~isempty(clipTubes)
	        dataSet = [dataSet; clipTubes];            
            midFrame = round((initFrame + finalFrame)/2);
            localIndices = repmat([i, midFrame], size(clipTubes,1), 1);
            localActions = repmat([{clipName}, {labelName}], size(clipTubes,1), 1);
            videoIndices = [videoIndices; localIndices];
            actionLabels = [actionLabels; localActions];
        end
        initFrame = finalFrame + 1;
    end
    if i == endIndex | strcmp(clipName, nextClipName)==0
		databaseName = [actionDir,'/', resultsDir, '/', 'database', strtok(clipName, '.')];
		save(databaseName, 'dataSet', 'videoIndices', 'actionLabels');
% 		databaseName = [actionDir,'/', resultsDir, '/', 'database', 'Test'];
%       save(databaseName, 'videoIndices', 'actionLabels');    
	end
    prevClipName = clipName;
end;


